# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetRealmResult',
    'AwaitableGetRealmResult',
    'get_realm',
    'get_realm_output',
]

@pulumi.output_type
class GetRealmResult:
    """
    A collection of values returned by getRealm.
    """
    def __init__(__self__, access_code_lifespan=None, access_code_lifespan_login=None, access_code_lifespan_user_action=None, access_token_lifespan=None, access_token_lifespan_for_implicit_flow=None, account_theme=None, action_token_generated_by_admin_lifespan=None, action_token_generated_by_user_lifespan=None, admin_theme=None, attributes=None, browser_flow=None, client_authentication_flow=None, client_session_idle_timeout=None, client_session_max_lifespan=None, default_default_client_scopes=None, default_optional_client_scopes=None, default_signature_algorithm=None, direct_grant_flow=None, display_name=None, display_name_html=None, docker_authentication_flow=None, duplicate_emails_allowed=None, edit_username_allowed=None, email_theme=None, enabled=None, id=None, internal_id=None, internationalizations=None, login_theme=None, login_with_email_allowed=None, oauth2_device_code_lifespan=None, oauth2_device_polling_interval=None, offline_session_idle_timeout=None, offline_session_max_lifespan=None, offline_session_max_lifespan_enabled=None, otp_policy=None, password_policy=None, realm=None, refresh_token_max_reuse=None, registration_allowed=None, registration_email_as_username=None, registration_flow=None, remember_me=None, reset_credentials_flow=None, reset_password_allowed=None, revoke_refresh_token=None, security_defenses=None, smtp_servers=None, ssl_required=None, sso_session_idle_timeout=None, sso_session_idle_timeout_remember_me=None, sso_session_max_lifespan=None, sso_session_max_lifespan_remember_me=None, user_managed_access=None, verify_email=None, web_authn_passwordless_policy=None, web_authn_policy=None):
        if access_code_lifespan and not isinstance(access_code_lifespan, str):
            raise TypeError("Expected argument 'access_code_lifespan' to be a str")
        pulumi.set(__self__, "access_code_lifespan", access_code_lifespan)
        if access_code_lifespan_login and not isinstance(access_code_lifespan_login, str):
            raise TypeError("Expected argument 'access_code_lifespan_login' to be a str")
        pulumi.set(__self__, "access_code_lifespan_login", access_code_lifespan_login)
        if access_code_lifespan_user_action and not isinstance(access_code_lifespan_user_action, str):
            raise TypeError("Expected argument 'access_code_lifespan_user_action' to be a str")
        pulumi.set(__self__, "access_code_lifespan_user_action", access_code_lifespan_user_action)
        if access_token_lifespan and not isinstance(access_token_lifespan, str):
            raise TypeError("Expected argument 'access_token_lifespan' to be a str")
        pulumi.set(__self__, "access_token_lifespan", access_token_lifespan)
        if access_token_lifespan_for_implicit_flow and not isinstance(access_token_lifespan_for_implicit_flow, str):
            raise TypeError("Expected argument 'access_token_lifespan_for_implicit_flow' to be a str")
        pulumi.set(__self__, "access_token_lifespan_for_implicit_flow", access_token_lifespan_for_implicit_flow)
        if account_theme and not isinstance(account_theme, str):
            raise TypeError("Expected argument 'account_theme' to be a str")
        pulumi.set(__self__, "account_theme", account_theme)
        if action_token_generated_by_admin_lifespan and not isinstance(action_token_generated_by_admin_lifespan, str):
            raise TypeError("Expected argument 'action_token_generated_by_admin_lifespan' to be a str")
        pulumi.set(__self__, "action_token_generated_by_admin_lifespan", action_token_generated_by_admin_lifespan)
        if action_token_generated_by_user_lifespan and not isinstance(action_token_generated_by_user_lifespan, str):
            raise TypeError("Expected argument 'action_token_generated_by_user_lifespan' to be a str")
        pulumi.set(__self__, "action_token_generated_by_user_lifespan", action_token_generated_by_user_lifespan)
        if admin_theme and not isinstance(admin_theme, str):
            raise TypeError("Expected argument 'admin_theme' to be a str")
        pulumi.set(__self__, "admin_theme", admin_theme)
        if attributes and not isinstance(attributes, dict):
            raise TypeError("Expected argument 'attributes' to be a dict")
        pulumi.set(__self__, "attributes", attributes)
        if browser_flow and not isinstance(browser_flow, str):
            raise TypeError("Expected argument 'browser_flow' to be a str")
        pulumi.set(__self__, "browser_flow", browser_flow)
        if client_authentication_flow and not isinstance(client_authentication_flow, str):
            raise TypeError("Expected argument 'client_authentication_flow' to be a str")
        pulumi.set(__self__, "client_authentication_flow", client_authentication_flow)
        if client_session_idle_timeout and not isinstance(client_session_idle_timeout, str):
            raise TypeError("Expected argument 'client_session_idle_timeout' to be a str")
        pulumi.set(__self__, "client_session_idle_timeout", client_session_idle_timeout)
        if client_session_max_lifespan and not isinstance(client_session_max_lifespan, str):
            raise TypeError("Expected argument 'client_session_max_lifespan' to be a str")
        pulumi.set(__self__, "client_session_max_lifespan", client_session_max_lifespan)
        if default_default_client_scopes and not isinstance(default_default_client_scopes, list):
            raise TypeError("Expected argument 'default_default_client_scopes' to be a list")
        pulumi.set(__self__, "default_default_client_scopes", default_default_client_scopes)
        if default_optional_client_scopes and not isinstance(default_optional_client_scopes, list):
            raise TypeError("Expected argument 'default_optional_client_scopes' to be a list")
        pulumi.set(__self__, "default_optional_client_scopes", default_optional_client_scopes)
        if default_signature_algorithm and not isinstance(default_signature_algorithm, str):
            raise TypeError("Expected argument 'default_signature_algorithm' to be a str")
        pulumi.set(__self__, "default_signature_algorithm", default_signature_algorithm)
        if direct_grant_flow and not isinstance(direct_grant_flow, str):
            raise TypeError("Expected argument 'direct_grant_flow' to be a str")
        pulumi.set(__self__, "direct_grant_flow", direct_grant_flow)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if display_name_html and not isinstance(display_name_html, str):
            raise TypeError("Expected argument 'display_name_html' to be a str")
        pulumi.set(__self__, "display_name_html", display_name_html)
        if docker_authentication_flow and not isinstance(docker_authentication_flow, str):
            raise TypeError("Expected argument 'docker_authentication_flow' to be a str")
        pulumi.set(__self__, "docker_authentication_flow", docker_authentication_flow)
        if duplicate_emails_allowed and not isinstance(duplicate_emails_allowed, bool):
            raise TypeError("Expected argument 'duplicate_emails_allowed' to be a bool")
        pulumi.set(__self__, "duplicate_emails_allowed", duplicate_emails_allowed)
        if edit_username_allowed and not isinstance(edit_username_allowed, bool):
            raise TypeError("Expected argument 'edit_username_allowed' to be a bool")
        pulumi.set(__self__, "edit_username_allowed", edit_username_allowed)
        if email_theme and not isinstance(email_theme, str):
            raise TypeError("Expected argument 'email_theme' to be a str")
        pulumi.set(__self__, "email_theme", email_theme)
        if enabled and not isinstance(enabled, bool):
            raise TypeError("Expected argument 'enabled' to be a bool")
        pulumi.set(__self__, "enabled", enabled)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if internal_id and not isinstance(internal_id, str):
            raise TypeError("Expected argument 'internal_id' to be a str")
        pulumi.set(__self__, "internal_id", internal_id)
        if internationalizations and not isinstance(internationalizations, list):
            raise TypeError("Expected argument 'internationalizations' to be a list")
        pulumi.set(__self__, "internationalizations", internationalizations)
        if login_theme and not isinstance(login_theme, str):
            raise TypeError("Expected argument 'login_theme' to be a str")
        pulumi.set(__self__, "login_theme", login_theme)
        if login_with_email_allowed and not isinstance(login_with_email_allowed, bool):
            raise TypeError("Expected argument 'login_with_email_allowed' to be a bool")
        pulumi.set(__self__, "login_with_email_allowed", login_with_email_allowed)
        if oauth2_device_code_lifespan and not isinstance(oauth2_device_code_lifespan, str):
            raise TypeError("Expected argument 'oauth2_device_code_lifespan' to be a str")
        pulumi.set(__self__, "oauth2_device_code_lifespan", oauth2_device_code_lifespan)
        if oauth2_device_polling_interval and not isinstance(oauth2_device_polling_interval, int):
            raise TypeError("Expected argument 'oauth2_device_polling_interval' to be a int")
        pulumi.set(__self__, "oauth2_device_polling_interval", oauth2_device_polling_interval)
        if offline_session_idle_timeout and not isinstance(offline_session_idle_timeout, str):
            raise TypeError("Expected argument 'offline_session_idle_timeout' to be a str")
        pulumi.set(__self__, "offline_session_idle_timeout", offline_session_idle_timeout)
        if offline_session_max_lifespan and not isinstance(offline_session_max_lifespan, str):
            raise TypeError("Expected argument 'offline_session_max_lifespan' to be a str")
        pulumi.set(__self__, "offline_session_max_lifespan", offline_session_max_lifespan)
        if offline_session_max_lifespan_enabled and not isinstance(offline_session_max_lifespan_enabled, bool):
            raise TypeError("Expected argument 'offline_session_max_lifespan_enabled' to be a bool")
        pulumi.set(__self__, "offline_session_max_lifespan_enabled", offline_session_max_lifespan_enabled)
        if otp_policy and not isinstance(otp_policy, dict):
            raise TypeError("Expected argument 'otp_policy' to be a dict")
        pulumi.set(__self__, "otp_policy", otp_policy)
        if password_policy and not isinstance(password_policy, str):
            raise TypeError("Expected argument 'password_policy' to be a str")
        pulumi.set(__self__, "password_policy", password_policy)
        if realm and not isinstance(realm, str):
            raise TypeError("Expected argument 'realm' to be a str")
        pulumi.set(__self__, "realm", realm)
        if refresh_token_max_reuse and not isinstance(refresh_token_max_reuse, int):
            raise TypeError("Expected argument 'refresh_token_max_reuse' to be a int")
        pulumi.set(__self__, "refresh_token_max_reuse", refresh_token_max_reuse)
        if registration_allowed and not isinstance(registration_allowed, bool):
            raise TypeError("Expected argument 'registration_allowed' to be a bool")
        pulumi.set(__self__, "registration_allowed", registration_allowed)
        if registration_email_as_username and not isinstance(registration_email_as_username, bool):
            raise TypeError("Expected argument 'registration_email_as_username' to be a bool")
        pulumi.set(__self__, "registration_email_as_username", registration_email_as_username)
        if registration_flow and not isinstance(registration_flow, str):
            raise TypeError("Expected argument 'registration_flow' to be a str")
        pulumi.set(__self__, "registration_flow", registration_flow)
        if remember_me and not isinstance(remember_me, bool):
            raise TypeError("Expected argument 'remember_me' to be a bool")
        pulumi.set(__self__, "remember_me", remember_me)
        if reset_credentials_flow and not isinstance(reset_credentials_flow, str):
            raise TypeError("Expected argument 'reset_credentials_flow' to be a str")
        pulumi.set(__self__, "reset_credentials_flow", reset_credentials_flow)
        if reset_password_allowed and not isinstance(reset_password_allowed, bool):
            raise TypeError("Expected argument 'reset_password_allowed' to be a bool")
        pulumi.set(__self__, "reset_password_allowed", reset_password_allowed)
        if revoke_refresh_token and not isinstance(revoke_refresh_token, bool):
            raise TypeError("Expected argument 'revoke_refresh_token' to be a bool")
        pulumi.set(__self__, "revoke_refresh_token", revoke_refresh_token)
        if security_defenses and not isinstance(security_defenses, list):
            raise TypeError("Expected argument 'security_defenses' to be a list")
        pulumi.set(__self__, "security_defenses", security_defenses)
        if smtp_servers and not isinstance(smtp_servers, list):
            raise TypeError("Expected argument 'smtp_servers' to be a list")
        pulumi.set(__self__, "smtp_servers", smtp_servers)
        if ssl_required and not isinstance(ssl_required, str):
            raise TypeError("Expected argument 'ssl_required' to be a str")
        pulumi.set(__self__, "ssl_required", ssl_required)
        if sso_session_idle_timeout and not isinstance(sso_session_idle_timeout, str):
            raise TypeError("Expected argument 'sso_session_idle_timeout' to be a str")
        pulumi.set(__self__, "sso_session_idle_timeout", sso_session_idle_timeout)
        if sso_session_idle_timeout_remember_me and not isinstance(sso_session_idle_timeout_remember_me, str):
            raise TypeError("Expected argument 'sso_session_idle_timeout_remember_me' to be a str")
        pulumi.set(__self__, "sso_session_idle_timeout_remember_me", sso_session_idle_timeout_remember_me)
        if sso_session_max_lifespan and not isinstance(sso_session_max_lifespan, str):
            raise TypeError("Expected argument 'sso_session_max_lifespan' to be a str")
        pulumi.set(__self__, "sso_session_max_lifespan", sso_session_max_lifespan)
        if sso_session_max_lifespan_remember_me and not isinstance(sso_session_max_lifespan_remember_me, str):
            raise TypeError("Expected argument 'sso_session_max_lifespan_remember_me' to be a str")
        pulumi.set(__self__, "sso_session_max_lifespan_remember_me", sso_session_max_lifespan_remember_me)
        if user_managed_access and not isinstance(user_managed_access, bool):
            raise TypeError("Expected argument 'user_managed_access' to be a bool")
        pulumi.set(__self__, "user_managed_access", user_managed_access)
        if verify_email and not isinstance(verify_email, bool):
            raise TypeError("Expected argument 'verify_email' to be a bool")
        pulumi.set(__self__, "verify_email", verify_email)
        if web_authn_passwordless_policy and not isinstance(web_authn_passwordless_policy, dict):
            raise TypeError("Expected argument 'web_authn_passwordless_policy' to be a dict")
        pulumi.set(__self__, "web_authn_passwordless_policy", web_authn_passwordless_policy)
        if web_authn_policy and not isinstance(web_authn_policy, dict):
            raise TypeError("Expected argument 'web_authn_policy' to be a dict")
        pulumi.set(__self__, "web_authn_policy", web_authn_policy)

    @property
    @pulumi.getter(name="accessCodeLifespan")
    def access_code_lifespan(self) -> str:
        return pulumi.get(self, "access_code_lifespan")

    @property
    @pulumi.getter(name="accessCodeLifespanLogin")
    def access_code_lifespan_login(self) -> str:
        return pulumi.get(self, "access_code_lifespan_login")

    @property
    @pulumi.getter(name="accessCodeLifespanUserAction")
    def access_code_lifespan_user_action(self) -> str:
        return pulumi.get(self, "access_code_lifespan_user_action")

    @property
    @pulumi.getter(name="accessTokenLifespan")
    def access_token_lifespan(self) -> str:
        return pulumi.get(self, "access_token_lifespan")

    @property
    @pulumi.getter(name="accessTokenLifespanForImplicitFlow")
    def access_token_lifespan_for_implicit_flow(self) -> str:
        return pulumi.get(self, "access_token_lifespan_for_implicit_flow")

    @property
    @pulumi.getter(name="accountTheme")
    def account_theme(self) -> str:
        return pulumi.get(self, "account_theme")

    @property
    @pulumi.getter(name="actionTokenGeneratedByAdminLifespan")
    def action_token_generated_by_admin_lifespan(self) -> str:
        return pulumi.get(self, "action_token_generated_by_admin_lifespan")

    @property
    @pulumi.getter(name="actionTokenGeneratedByUserLifespan")
    def action_token_generated_by_user_lifespan(self) -> str:
        return pulumi.get(self, "action_token_generated_by_user_lifespan")

    @property
    @pulumi.getter(name="adminTheme")
    def admin_theme(self) -> str:
        return pulumi.get(self, "admin_theme")

    @property
    @pulumi.getter
    def attributes(self) -> Mapping[str, Any]:
        return pulumi.get(self, "attributes")

    @property
    @pulumi.getter(name="browserFlow")
    def browser_flow(self) -> str:
        return pulumi.get(self, "browser_flow")

    @property
    @pulumi.getter(name="clientAuthenticationFlow")
    def client_authentication_flow(self) -> str:
        return pulumi.get(self, "client_authentication_flow")

    @property
    @pulumi.getter(name="clientSessionIdleTimeout")
    def client_session_idle_timeout(self) -> str:
        return pulumi.get(self, "client_session_idle_timeout")

    @property
    @pulumi.getter(name="clientSessionMaxLifespan")
    def client_session_max_lifespan(self) -> str:
        return pulumi.get(self, "client_session_max_lifespan")

    @property
    @pulumi.getter(name="defaultDefaultClientScopes")
    def default_default_client_scopes(self) -> Sequence[str]:
        return pulumi.get(self, "default_default_client_scopes")

    @property
    @pulumi.getter(name="defaultOptionalClientScopes")
    def default_optional_client_scopes(self) -> Sequence[str]:
        return pulumi.get(self, "default_optional_client_scopes")

    @property
    @pulumi.getter(name="defaultSignatureAlgorithm")
    def default_signature_algorithm(self) -> str:
        return pulumi.get(self, "default_signature_algorithm")

    @property
    @pulumi.getter(name="directGrantFlow")
    def direct_grant_flow(self) -> str:
        return pulumi.get(self, "direct_grant_flow")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="displayNameHtml")
    def display_name_html(self) -> Optional[str]:
        return pulumi.get(self, "display_name_html")

    @property
    @pulumi.getter(name="dockerAuthenticationFlow")
    def docker_authentication_flow(self) -> str:
        return pulumi.get(self, "docker_authentication_flow")

    @property
    @pulumi.getter(name="duplicateEmailsAllowed")
    def duplicate_emails_allowed(self) -> bool:
        return pulumi.get(self, "duplicate_emails_allowed")

    @property
    @pulumi.getter(name="editUsernameAllowed")
    def edit_username_allowed(self) -> bool:
        return pulumi.get(self, "edit_username_allowed")

    @property
    @pulumi.getter(name="emailTheme")
    def email_theme(self) -> str:
        return pulumi.get(self, "email_theme")

    @property
    @pulumi.getter
    def enabled(self) -> bool:
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="internalId")
    def internal_id(self) -> str:
        return pulumi.get(self, "internal_id")

    @property
    @pulumi.getter
    def internationalizations(self) -> Sequence['outputs.GetRealmInternationalizationResult']:
        return pulumi.get(self, "internationalizations")

    @property
    @pulumi.getter(name="loginTheme")
    def login_theme(self) -> str:
        return pulumi.get(self, "login_theme")

    @property
    @pulumi.getter(name="loginWithEmailAllowed")
    def login_with_email_allowed(self) -> bool:
        return pulumi.get(self, "login_with_email_allowed")

    @property
    @pulumi.getter(name="oauth2DeviceCodeLifespan")
    def oauth2_device_code_lifespan(self) -> str:
        return pulumi.get(self, "oauth2_device_code_lifespan")

    @property
    @pulumi.getter(name="oauth2DevicePollingInterval")
    def oauth2_device_polling_interval(self) -> int:
        return pulumi.get(self, "oauth2_device_polling_interval")

    @property
    @pulumi.getter(name="offlineSessionIdleTimeout")
    def offline_session_idle_timeout(self) -> str:
        return pulumi.get(self, "offline_session_idle_timeout")

    @property
    @pulumi.getter(name="offlineSessionMaxLifespan")
    def offline_session_max_lifespan(self) -> str:
        return pulumi.get(self, "offline_session_max_lifespan")

    @property
    @pulumi.getter(name="offlineSessionMaxLifespanEnabled")
    def offline_session_max_lifespan_enabled(self) -> bool:
        return pulumi.get(self, "offline_session_max_lifespan_enabled")

    @property
    @pulumi.getter(name="otpPolicy")
    def otp_policy(self) -> 'outputs.GetRealmOtpPolicyResult':
        return pulumi.get(self, "otp_policy")

    @property
    @pulumi.getter(name="passwordPolicy")
    def password_policy(self) -> str:
        return pulumi.get(self, "password_policy")

    @property
    @pulumi.getter
    def realm(self) -> str:
        return pulumi.get(self, "realm")

    @property
    @pulumi.getter(name="refreshTokenMaxReuse")
    def refresh_token_max_reuse(self) -> int:
        return pulumi.get(self, "refresh_token_max_reuse")

    @property
    @pulumi.getter(name="registrationAllowed")
    def registration_allowed(self) -> bool:
        return pulumi.get(self, "registration_allowed")

    @property
    @pulumi.getter(name="registrationEmailAsUsername")
    def registration_email_as_username(self) -> bool:
        return pulumi.get(self, "registration_email_as_username")

    @property
    @pulumi.getter(name="registrationFlow")
    def registration_flow(self) -> str:
        return pulumi.get(self, "registration_flow")

    @property
    @pulumi.getter(name="rememberMe")
    def remember_me(self) -> bool:
        return pulumi.get(self, "remember_me")

    @property
    @pulumi.getter(name="resetCredentialsFlow")
    def reset_credentials_flow(self) -> str:
        return pulumi.get(self, "reset_credentials_flow")

    @property
    @pulumi.getter(name="resetPasswordAllowed")
    def reset_password_allowed(self) -> bool:
        return pulumi.get(self, "reset_password_allowed")

    @property
    @pulumi.getter(name="revokeRefreshToken")
    def revoke_refresh_token(self) -> bool:
        return pulumi.get(self, "revoke_refresh_token")

    @property
    @pulumi.getter(name="securityDefenses")
    def security_defenses(self) -> Sequence['outputs.GetRealmSecurityDefenseResult']:
        return pulumi.get(self, "security_defenses")

    @property
    @pulumi.getter(name="smtpServers")
    def smtp_servers(self) -> Sequence['outputs.GetRealmSmtpServerResult']:
        return pulumi.get(self, "smtp_servers")

    @property
    @pulumi.getter(name="sslRequired")
    def ssl_required(self) -> str:
        return pulumi.get(self, "ssl_required")

    @property
    @pulumi.getter(name="ssoSessionIdleTimeout")
    def sso_session_idle_timeout(self) -> str:
        return pulumi.get(self, "sso_session_idle_timeout")

    @property
    @pulumi.getter(name="ssoSessionIdleTimeoutRememberMe")
    def sso_session_idle_timeout_remember_me(self) -> str:
        return pulumi.get(self, "sso_session_idle_timeout_remember_me")

    @property
    @pulumi.getter(name="ssoSessionMaxLifespan")
    def sso_session_max_lifespan(self) -> str:
        return pulumi.get(self, "sso_session_max_lifespan")

    @property
    @pulumi.getter(name="ssoSessionMaxLifespanRememberMe")
    def sso_session_max_lifespan_remember_me(self) -> str:
        return pulumi.get(self, "sso_session_max_lifespan_remember_me")

    @property
    @pulumi.getter(name="userManagedAccess")
    def user_managed_access(self) -> bool:
        return pulumi.get(self, "user_managed_access")

    @property
    @pulumi.getter(name="verifyEmail")
    def verify_email(self) -> bool:
        return pulumi.get(self, "verify_email")

    @property
    @pulumi.getter(name="webAuthnPasswordlessPolicy")
    def web_authn_passwordless_policy(self) -> 'outputs.GetRealmWebAuthnPasswordlessPolicyResult':
        return pulumi.get(self, "web_authn_passwordless_policy")

    @property
    @pulumi.getter(name="webAuthnPolicy")
    def web_authn_policy(self) -> 'outputs.GetRealmWebAuthnPolicyResult':
        return pulumi.get(self, "web_authn_policy")


class AwaitableGetRealmResult(GetRealmResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetRealmResult(
            access_code_lifespan=self.access_code_lifespan,
            access_code_lifespan_login=self.access_code_lifespan_login,
            access_code_lifespan_user_action=self.access_code_lifespan_user_action,
            access_token_lifespan=self.access_token_lifespan,
            access_token_lifespan_for_implicit_flow=self.access_token_lifespan_for_implicit_flow,
            account_theme=self.account_theme,
            action_token_generated_by_admin_lifespan=self.action_token_generated_by_admin_lifespan,
            action_token_generated_by_user_lifespan=self.action_token_generated_by_user_lifespan,
            admin_theme=self.admin_theme,
            attributes=self.attributes,
            browser_flow=self.browser_flow,
            client_authentication_flow=self.client_authentication_flow,
            client_session_idle_timeout=self.client_session_idle_timeout,
            client_session_max_lifespan=self.client_session_max_lifespan,
            default_default_client_scopes=self.default_default_client_scopes,
            default_optional_client_scopes=self.default_optional_client_scopes,
            default_signature_algorithm=self.default_signature_algorithm,
            direct_grant_flow=self.direct_grant_flow,
            display_name=self.display_name,
            display_name_html=self.display_name_html,
            docker_authentication_flow=self.docker_authentication_flow,
            duplicate_emails_allowed=self.duplicate_emails_allowed,
            edit_username_allowed=self.edit_username_allowed,
            email_theme=self.email_theme,
            enabled=self.enabled,
            id=self.id,
            internal_id=self.internal_id,
            internationalizations=self.internationalizations,
            login_theme=self.login_theme,
            login_with_email_allowed=self.login_with_email_allowed,
            oauth2_device_code_lifespan=self.oauth2_device_code_lifespan,
            oauth2_device_polling_interval=self.oauth2_device_polling_interval,
            offline_session_idle_timeout=self.offline_session_idle_timeout,
            offline_session_max_lifespan=self.offline_session_max_lifespan,
            offline_session_max_lifespan_enabled=self.offline_session_max_lifespan_enabled,
            otp_policy=self.otp_policy,
            password_policy=self.password_policy,
            realm=self.realm,
            refresh_token_max_reuse=self.refresh_token_max_reuse,
            registration_allowed=self.registration_allowed,
            registration_email_as_username=self.registration_email_as_username,
            registration_flow=self.registration_flow,
            remember_me=self.remember_me,
            reset_credentials_flow=self.reset_credentials_flow,
            reset_password_allowed=self.reset_password_allowed,
            revoke_refresh_token=self.revoke_refresh_token,
            security_defenses=self.security_defenses,
            smtp_servers=self.smtp_servers,
            ssl_required=self.ssl_required,
            sso_session_idle_timeout=self.sso_session_idle_timeout,
            sso_session_idle_timeout_remember_me=self.sso_session_idle_timeout_remember_me,
            sso_session_max_lifespan=self.sso_session_max_lifespan,
            sso_session_max_lifespan_remember_me=self.sso_session_max_lifespan_remember_me,
            user_managed_access=self.user_managed_access,
            verify_email=self.verify_email,
            web_authn_passwordless_policy=self.web_authn_passwordless_policy,
            web_authn_policy=self.web_authn_policy)


def get_realm(attributes: Optional[Mapping[str, Any]] = None,
              default_default_client_scopes: Optional[Sequence[str]] = None,
              default_optional_client_scopes: Optional[Sequence[str]] = None,
              display_name_html: Optional[str] = None,
              internationalizations: Optional[Sequence[pulumi.InputType['GetRealmInternationalizationArgs']]] = None,
              otp_policy: Optional[pulumi.InputType['GetRealmOtpPolicyArgs']] = None,
              realm: Optional[str] = None,
              security_defenses: Optional[Sequence[pulumi.InputType['GetRealmSecurityDefenseArgs']]] = None,
              smtp_servers: Optional[Sequence[pulumi.InputType['GetRealmSmtpServerArgs']]] = None,
              web_authn_passwordless_policy: Optional[pulumi.InputType['GetRealmWebAuthnPasswordlessPolicyArgs']] = None,
              web_authn_policy: Optional[pulumi.InputType['GetRealmWebAuthnPolicyArgs']] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetRealmResult:
    """
    This data source can be used to fetch properties of a Keycloak realm for
    usage with other resources.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_keycloak as keycloak

    realm = keycloak.get_realm(realm="my-realm")
    group = keycloak.Role("group", realm_id=realm.id)
    ```


    :param str realm: The realm name.
    """
    __args__ = dict()
    __args__['attributes'] = attributes
    __args__['defaultDefaultClientScopes'] = default_default_client_scopes
    __args__['defaultOptionalClientScopes'] = default_optional_client_scopes
    __args__['displayNameHtml'] = display_name_html
    __args__['internationalizations'] = internationalizations
    __args__['otpPolicy'] = otp_policy
    __args__['realm'] = realm
    __args__['securityDefenses'] = security_defenses
    __args__['smtpServers'] = smtp_servers
    __args__['webAuthnPasswordlessPolicy'] = web_authn_passwordless_policy
    __args__['webAuthnPolicy'] = web_authn_policy
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = _utilities.get_version()
    __ret__ = pulumi.runtime.invoke('keycloak:index/getRealm:getRealm', __args__, opts=opts, typ=GetRealmResult).value

    return AwaitableGetRealmResult(
        access_code_lifespan=__ret__.access_code_lifespan,
        access_code_lifespan_login=__ret__.access_code_lifespan_login,
        access_code_lifespan_user_action=__ret__.access_code_lifespan_user_action,
        access_token_lifespan=__ret__.access_token_lifespan,
        access_token_lifespan_for_implicit_flow=__ret__.access_token_lifespan_for_implicit_flow,
        account_theme=__ret__.account_theme,
        action_token_generated_by_admin_lifespan=__ret__.action_token_generated_by_admin_lifespan,
        action_token_generated_by_user_lifespan=__ret__.action_token_generated_by_user_lifespan,
        admin_theme=__ret__.admin_theme,
        attributes=__ret__.attributes,
        browser_flow=__ret__.browser_flow,
        client_authentication_flow=__ret__.client_authentication_flow,
        client_session_idle_timeout=__ret__.client_session_idle_timeout,
        client_session_max_lifespan=__ret__.client_session_max_lifespan,
        default_default_client_scopes=__ret__.default_default_client_scopes,
        default_optional_client_scopes=__ret__.default_optional_client_scopes,
        default_signature_algorithm=__ret__.default_signature_algorithm,
        direct_grant_flow=__ret__.direct_grant_flow,
        display_name=__ret__.display_name,
        display_name_html=__ret__.display_name_html,
        docker_authentication_flow=__ret__.docker_authentication_flow,
        duplicate_emails_allowed=__ret__.duplicate_emails_allowed,
        edit_username_allowed=__ret__.edit_username_allowed,
        email_theme=__ret__.email_theme,
        enabled=__ret__.enabled,
        id=__ret__.id,
        internal_id=__ret__.internal_id,
        internationalizations=__ret__.internationalizations,
        login_theme=__ret__.login_theme,
        login_with_email_allowed=__ret__.login_with_email_allowed,
        oauth2_device_code_lifespan=__ret__.oauth2_device_code_lifespan,
        oauth2_device_polling_interval=__ret__.oauth2_device_polling_interval,
        offline_session_idle_timeout=__ret__.offline_session_idle_timeout,
        offline_session_max_lifespan=__ret__.offline_session_max_lifespan,
        offline_session_max_lifespan_enabled=__ret__.offline_session_max_lifespan_enabled,
        otp_policy=__ret__.otp_policy,
        password_policy=__ret__.password_policy,
        realm=__ret__.realm,
        refresh_token_max_reuse=__ret__.refresh_token_max_reuse,
        registration_allowed=__ret__.registration_allowed,
        registration_email_as_username=__ret__.registration_email_as_username,
        registration_flow=__ret__.registration_flow,
        remember_me=__ret__.remember_me,
        reset_credentials_flow=__ret__.reset_credentials_flow,
        reset_password_allowed=__ret__.reset_password_allowed,
        revoke_refresh_token=__ret__.revoke_refresh_token,
        security_defenses=__ret__.security_defenses,
        smtp_servers=__ret__.smtp_servers,
        ssl_required=__ret__.ssl_required,
        sso_session_idle_timeout=__ret__.sso_session_idle_timeout,
        sso_session_idle_timeout_remember_me=__ret__.sso_session_idle_timeout_remember_me,
        sso_session_max_lifespan=__ret__.sso_session_max_lifespan,
        sso_session_max_lifespan_remember_me=__ret__.sso_session_max_lifespan_remember_me,
        user_managed_access=__ret__.user_managed_access,
        verify_email=__ret__.verify_email,
        web_authn_passwordless_policy=__ret__.web_authn_passwordless_policy,
        web_authn_policy=__ret__.web_authn_policy)


@_utilities.lift_output_func(get_realm)
def get_realm_output(attributes: Optional[pulumi.Input[Optional[Mapping[str, Any]]]] = None,
                     default_default_client_scopes: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                     default_optional_client_scopes: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                     display_name_html: Optional[pulumi.Input[Optional[str]]] = None,
                     internationalizations: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetRealmInternationalizationArgs']]]]] = None,
                     otp_policy: Optional[pulumi.Input[Optional[pulumi.InputType['GetRealmOtpPolicyArgs']]]] = None,
                     realm: Optional[pulumi.Input[str]] = None,
                     security_defenses: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetRealmSecurityDefenseArgs']]]]] = None,
                     smtp_servers: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetRealmSmtpServerArgs']]]]] = None,
                     web_authn_passwordless_policy: Optional[pulumi.Input[Optional[pulumi.InputType['GetRealmWebAuthnPasswordlessPolicyArgs']]]] = None,
                     web_authn_policy: Optional[pulumi.Input[Optional[pulumi.InputType['GetRealmWebAuthnPolicyArgs']]]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetRealmResult]:
    """
    This data source can be used to fetch properties of a Keycloak realm for
    usage with other resources.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_keycloak as keycloak

    realm = keycloak.get_realm(realm="my-realm")
    group = keycloak.Role("group", realm_id=realm.id)
    ```


    :param str realm: The realm name.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'UserFederationCacheArgs',
    'UserFederationKerberosArgs',
]

@pulumi.input_type
class UserFederationCacheArgs:
    def __init__(__self__, *,
                 eviction_day: Optional[pulumi.Input[int]] = None,
                 eviction_hour: Optional[pulumi.Input[int]] = None,
                 eviction_minute: Optional[pulumi.Input[int]] = None,
                 max_lifespan: Optional[pulumi.Input[str]] = None,
                 policy: Optional[pulumi.Input[str]] = None):
        """
        :param pulumi.Input[int] eviction_day: Day of the week the entry will become invalid on
        :param pulumi.Input[int] eviction_hour: Hour of day the entry will become invalid on.
        :param pulumi.Input[int] eviction_minute: Minute of day the entry will become invalid on.
        :param pulumi.Input[str] max_lifespan: Max lifespan of cache entry (duration string).
        :param pulumi.Input[str] policy: Can be one of `DEFAULT`, `EVICT_DAILY`, `EVICT_WEEKLY`, `MAX_LIFESPAN`, or `NO_CACHE`. Defaults to `DEFAULT`.
        """
        if eviction_day is not None:
            pulumi.set(__self__, "eviction_day", eviction_day)
        if eviction_hour is not None:
            pulumi.set(__self__, "eviction_hour", eviction_hour)
        if eviction_minute is not None:
            pulumi.set(__self__, "eviction_minute", eviction_minute)
        if max_lifespan is not None:
            pulumi.set(__self__, "max_lifespan", max_lifespan)
        if policy is not None:
            pulumi.set(__self__, "policy", policy)

    @property
    @pulumi.getter(name="evictionDay")
    def eviction_day(self) -> Optional[pulumi.Input[int]]:
        """
        Day of the week the entry will become invalid on
        """
        return pulumi.get(self, "eviction_day")

    @eviction_day.setter
    def eviction_day(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "eviction_day", value)

    @property
    @pulumi.getter(name="evictionHour")
    def eviction_hour(self) -> Optional[pulumi.Input[int]]:
        """
        Hour of day the entry will become invalid on.
        """
        return pulumi.get(self, "eviction_hour")

    @eviction_hour.setter
    def eviction_hour(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "eviction_hour", value)

    @property
    @pulumi.getter(name="evictionMinute")
    def eviction_minute(self) -> Optional[pulumi.Input[int]]:
        """
        Minute of day the entry will become invalid on.
        """
        return pulumi.get(self, "eviction_minute")

    @eviction_minute.setter
    def eviction_minute(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "eviction_minute", value)

    @property
    @pulumi.getter(name="maxLifespan")
    def max_lifespan(self) -> Optional[pulumi.Input[str]]:
        """
        Max lifespan of cache entry (duration string).
        """
        return pulumi.get(self, "max_lifespan")

    @max_lifespan.setter
    def max_lifespan(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "max_lifespan", value)

    @property
    @pulumi.getter
    def policy(self) -> Optional[pulumi.Input[str]]:
        """
        Can be one of `DEFAULT`, `EVICT_DAILY`, `EVICT_WEEKLY`, `MAX_LIFESPAN`, or `NO_CACHE`. Defaults to `DEFAULT`.
        """
        return pulumi.get(self, "policy")

    @policy.setter
    def policy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "policy", value)


@pulumi.input_type
class UserFederationKerberosArgs:
    def __init__(__self__, *,
                 kerberos_realm: pulumi.Input[str],
                 key_tab: pulumi.Input[str],
                 server_principal: pulumi.Input[str],
                 use_kerberos_for_password_authentication: Optional[pulumi.Input[bool]] = None):
        """
        :param pulumi.Input[str] kerberos_realm: The name of the kerberos realm, e.g. FOO.LOCAL.
        :param pulumi.Input[str] key_tab: Path to the kerberos keytab file on the server with credentials of the service principal.
        :param pulumi.Input[str] server_principal: The kerberos server principal, e.g. 'HTTP/host.foo.com@FOO.LOCAL'.
        :param pulumi.Input[bool] use_kerberos_for_password_authentication: Use kerberos login module instead of ldap service api. Defaults to `false`.
        """
        pulumi.set(__self__, "kerberos_realm", kerberos_realm)
        pulumi.set(__self__, "key_tab", key_tab)
        pulumi.set(__self__, "server_principal", server_principal)
        if use_kerberos_for_password_authentication is not None:
            pulumi.set(__self__, "use_kerberos_for_password_authentication", use_kerberos_for_password_authentication)

    @property
    @pulumi.getter(name="kerberosRealm")
    def kerberos_realm(self) -> pulumi.Input[str]:
        """
        The name of the kerberos realm, e.g. FOO.LOCAL.
        """
        return pulumi.get(self, "kerberos_realm")

    @kerberos_realm.setter
    def kerberos_realm(self, value: pulumi.Input[str]):
        pulumi.set(self, "kerberos_realm", value)

    @property
    @pulumi.getter(name="keyTab")
    def key_tab(self) -> pulumi.Input[str]:
        """
        Path to the kerberos keytab file on the server with credentials of the service principal.
        """
        return pulumi.get(self, "key_tab")

    @key_tab.setter
    def key_tab(self, value: pulumi.Input[str]):
        pulumi.set(self, "key_tab", value)

    @property
    @pulumi.getter(name="serverPrincipal")
    def server_principal(self) -> pulumi.Input[str]:
        """
        The kerberos server principal, e.g. 'HTTP/host.foo.com@FOO.LOCAL'.
        """
        return pulumi.get(self, "server_principal")

    @server_principal.setter
    def server_principal(self, value: pulumi.Input[str]):
        pulumi.set(self, "server_principal", value)

    @property
    @pulumi.getter(name="useKerberosForPasswordAuthentication")
    def use_kerberos_for_password_authentication(self) -> Optional[pulumi.Input[bool]]:
        """
        Use kerberos login module instead of ldap service api. Defaults to `false`.
        """
        return pulumi.get(self, "use_kerberos_for_password_authentication")

    @use_kerberos_for_password_authentication.setter
    def use_kerberos_for_password_authentication(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "use_kerberos_for_password_authentication", value)





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

import types

__config__ = pulumi.Config('keycloak')


class _ExportableConfig(types.ModuleType):
    @property
    def additional_headers(self) -> Optional[str]:
        return __config__.get('additionalHeaders')

    @property
    def base_path(self) -> Optional[str]:
        return __config__.get('basePath')

    @property
    def client_id(self) -> Optional[str]:
        return __config__.get('clientId')

    @property
    def client_secret(self) -> Optional[str]:
        return __config__.get('clientSecret')

    @property
    def client_timeout(self) -> int:
        """
        Timeout (in seconds) of the Keycloak client
        """
        return __config__.get_int('clientTimeout') or (_utilities.get_env_int('KEYCLOAK_CLIENT_TIMEOUT') or 5)

    @property
    def initial_login(self) -> Optional[bool]:
        """
        Whether or not to login to Keycloak instance on provider initialization
        """
        return __config__.get_bool('initialLogin')

    @property
    def password(self) -> Optional[str]:
        return __config__.get('password')

    @property
    def realm(self) -> Optional[str]:
        return __config__.get('realm')

    @property
    def root_ca_certificate(self) -> Optional[str]:
        """
        Allows x509 calls using an unknown CA certificate (for development purposes)
        """
        return __config__.get('rootCaCertificate')

    @property
    def tls_insecure_skip_verify(self) -> Optional[bool]:
        """
        Allows ignoring insecure certificates when set to true. Defaults to false. Disabling security check is dangerous and
        should be avoided.
        """
        return __config__.get_bool('tlsInsecureSkipVerify')

    @property
    def url(self) -> Optional[str]:
        """
        The base URL of the Keycloak instance, before `/auth`
        """
        return __config__.get('url')

    @property
    def username(self) -> Optional[str]:
        return __config__.get('username')




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetClientServiceAccountUserResult',
    'AwaitableGetClientServiceAccountUserResult',
    'get_client_service_account_user',
    'get_client_service_account_user_output',
]

@pulumi.output_type
class GetClientServiceAccountUserResult:
    """
    A collection of values returned by getClientServiceAccountUser.
    """
    def __init__(__self__, attributes=None, client_id=None, email=None, email_verified=None, enabled=None, federated_identities=None, first_name=None, id=None, last_name=None, realm_id=None, username=None):
        if attributes and not isinstance(attributes, dict):
            raise TypeError("Expected argument 'attributes' to be a dict")
        pulumi.set(__self__, "attributes", attributes)
        if client_id and not isinstance(client_id, str):
            raise TypeError("Expected argument 'client_id' to be a str")
        pulumi.set(__self__, "client_id", client_id)
        if email and not isinstance(email, str):
            raise TypeError("Expected argument 'email' to be a str")
        pulumi.set(__self__, "email", email)
        if email_verified and not isinstance(email_verified, bool):
            raise TypeError("Expected argument 'email_verified' to be a bool")
        pulumi.set(__self__, "email_verified", email_verified)
        if enabled and not isinstance(enabled, bool):
            raise TypeError("Expected argument 'enabled' to be a bool")
        pulumi.set(__self__, "enabled", enabled)
        if federated_identities and not isinstance(federated_identities, list):
            raise TypeError("Expected argument 'federated_identities' to be a list")
        pulumi.set(__self__, "federated_identities", federated_identities)
        if first_name and not isinstance(first_name, str):
            raise TypeError("Expected argument 'first_name' to be a str")
        pulumi.set(__self__, "first_name", first_name)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if last_name and not isinstance(last_name, str):
            raise TypeError("Expected argument 'last_name' to be a str")
        pulumi.set(__self__, "last_name", last_name)
        if realm_id and not isinstance(realm_id, str):
            raise TypeError("Expected argument 'realm_id' to be a str")
        pulumi.set(__self__, "realm_id", realm_id)
        if username and not isinstance(username, str):
            raise TypeError("Expected argument 'username' to be a str")
        pulumi.set(__self__, "username", username)

    @property
    @pulumi.getter
    def attributes(self) -> Mapping[str, Any]:
        return pulumi.get(self, "attributes")

    @property
    @pulumi.getter(name="clientId")
    def client_id(self) -> str:
        return pulumi.get(self, "client_id")

    @property
    @pulumi.getter
    def email(self) -> str:
        return pulumi.get(self, "email")

    @property
    @pulumi.getter(name="emailVerified")
    def email_verified(self) -> bool:
        return pulumi.get(self, "email_verified")

    @property
    @pulumi.getter
    def enabled(self) -> bool:
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter(name="federatedIdentities")
    def federated_identities(self) -> Sequence['outputs.GetClientServiceAccountUserFederatedIdentityResult']:
        return pulumi.get(self, "federated_identities")

    @property
    @pulumi.getter(name="firstName")
    def first_name(self) -> str:
        return pulumi.get(self, "first_name")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="lastName")
    def last_name(self) -> str:
        return pulumi.get(self, "last_name")

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> str:
        return pulumi.get(self, "realm_id")

    @property
    @pulumi.getter
    def username(self) -> str:
        return pulumi.get(self, "username")


class AwaitableGetClientServiceAccountUserResult(GetClientServiceAccountUserResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetClientServiceAccountUserResult(
            attributes=self.attributes,
            client_id=self.client_id,
            email=self.email,
            email_verified=self.email_verified,
            enabled=self.enabled,
            federated_identities=self.federated_identities,
            first_name=self.first_name,
            id=self.id,
            last_name=self.last_name,
            realm_id=self.realm_id,
            username=self.username)


def get_client_service_account_user(client_id: Optional[str] = None,
                                    realm_id: Optional[str] = None,
                                    opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetClientServiceAccountUserResult:
    """
    This data source can be used to fetch information about the service account user that is associated with an OpenID client
    that has service accounts enabled.

    ## Example Usage

    In this example, we'll create an OpenID client with service accounts enabled. This causes Keycloak to create a special user
    that represents the service account. We'll use this data source to grab this user's ID in order to assign some roles to this
    user, using the `UserRoles` resource.

    ```python
    import pulumi
    import pulumi_keycloak as keycloak

    realm = keycloak.Realm("realm",
        realm="my-realm",
        enabled=True)
    client = keycloak.openid.Client("client",
        realm_id=realm.id,
        client_id="client",
        access_type="CONFIDENTIAL",
        service_accounts_enabled=True)
    service_account_user = keycloak.openid.get_client_service_account_user_output(realm_id=realm.id,
        client_id=client.id)
    offline_access = keycloak.get_role_output(realm_id=realm.id,
        name="offline_access")
    service_account_user_roles = keycloak.UserRoles("serviceAccountUserRoles",
        realm_id=realm.id,
        user_id=service_account_user.id,
        role_ids=[offline_access.id])
    ```


    :param str client_id: The ID of the OpenID client with service accounts enabled.
    :param str realm_id: The realm that the OpenID client exists within.
    """
    __args__ = dict()
    __args__['clientId'] = client_id
    __args__['realmId'] = realm_id
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = _utilities.get_version()
    __ret__ = pulumi.runtime.invoke('keycloak:openid/getClientServiceAccountUser:getClientServiceAccountUser', __args__, opts=opts, typ=GetClientServiceAccountUserResult).value

    return AwaitableGetClientServiceAccountUserResult(
        attributes=__ret__.attributes,
        client_id=__ret__.client_id,
        email=__ret__.email,
        email_verified=__ret__.email_verified,
        enabled=__ret__.enabled,
        federated_identities=__ret__.federated_identities,
        first_name=__ret__.first_name,
        id=__ret__.id,
        last_name=__ret__.last_name,
        realm_id=__ret__.realm_id,
        username=__ret__.username)


@_utilities.lift_output_func(get_client_service_account_user)
def get_client_service_account_user_output(client_id: Optional[pulumi.Input[str]] = None,
                                           realm_id: Optional[pulumi.Input[str]] = None,
                                           opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetClientServiceAccountUserResult]:
    """
    This data source can be used to fetch information about the service account user that is associated with an OpenID client
    that has service accounts enabled.

    ## Example Usage

    In this example, we'll create an OpenID client with service accounts enabled. This causes Keycloak to create a special user
    that represents the service account. We'll use this data source to grab this user's ID in order to assign some roles to this
    user, using the `UserRoles` resource.

    ```python
    import pulumi
    import pulumi_keycloak as keycloak

    realm = keycloak.Realm("realm",
        realm="my-realm",
        enabled=True)
    client = keycloak.openid.Client("client",
        realm_id=realm.id,
        client_id="client",
        access_type="CONFIDENTIAL",
        service_accounts_enabled=True)
    service_account_user = keycloak.openid.get_client_service_account_user_output(realm_id=realm.id,
        client_id=client.id)
    offline_access = keycloak.get_role_output(realm_id=realm.id,
        name="offline_access")
    service_account_user_roles = keycloak.UserRoles("serviceAccountUserRoles",
        realm_id=realm.id,
        user_id=service_account_user.id,
        role_ids=[offline_access.id])
    ```


    :param str client_id: The ID of the OpenID client with service accounts enabled.
    :param str realm_id: The realm that the OpenID client exists within.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['ExecutionArgs', 'Execution']

@pulumi.input_type
class ExecutionArgs:
    def __init__(__self__, *,
                 authenticator: pulumi.Input[str],
                 parent_flow_alias: pulumi.Input[str],
                 realm_id: pulumi.Input[str],
                 requirement: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Execution resource.
        :param pulumi.Input[str] authenticator: The name of the authenticator. This can be found by experimenting with the GUI and looking at HTTP requests within the network tab of your browser's development tools.
        :param pulumi.Input[str] parent_flow_alias: The alias of the flow this execution is attached to.
        :param pulumi.Input[str] realm_id: The realm the authentication execution exists in.
        :param pulumi.Input[str] requirement: The requirement setting, which can be one of `REQUIRED`, `ALTERNATIVE`, `OPTIONAL`, `CONDITIONAL`, or `DISABLED`. Defaults to `DISABLED`.
        """
        pulumi.set(__self__, "authenticator", authenticator)
        pulumi.set(__self__, "parent_flow_alias", parent_flow_alias)
        pulumi.set(__self__, "realm_id", realm_id)
        if requirement is not None:
            pulumi.set(__self__, "requirement", requirement)

    @property
    @pulumi.getter
    def authenticator(self) -> pulumi.Input[str]:
        """
        The name of the authenticator. This can be found by experimenting with the GUI and looking at HTTP requests within the network tab of your browser's development tools.
        """
        return pulumi.get(self, "authenticator")

    @authenticator.setter
    def authenticator(self, value: pulumi.Input[str]):
        pulumi.set(self, "authenticator", value)

    @property
    @pulumi.getter(name="parentFlowAlias")
    def parent_flow_alias(self) -> pulumi.Input[str]:
        """
        The alias of the flow this execution is attached to.
        """
        return pulumi.get(self, "parent_flow_alias")

    @parent_flow_alias.setter
    def parent_flow_alias(self, value: pulumi.Input[str]):
        pulumi.set(self, "parent_flow_alias", value)

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> pulumi.Input[str]:
        """
        The realm the authentication execution exists in.
        """
        return pulumi.get(self, "realm_id")

    @realm_id.setter
    def realm_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "realm_id", value)

    @property
    @pulumi.getter
    def requirement(self) -> Optional[pulumi.Input[str]]:
        """
        The requirement setting, which can be one of `REQUIRED`, `ALTERNATIVE`, `OPTIONAL`, `CONDITIONAL`, or `DISABLED`. Defaults to `DISABLED`.
        """
        return pulumi.get(self, "requirement")

    @requirement.setter
    def requirement(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "requirement", value)


@pulumi.input_type
class _ExecutionState:
    def __init__(__self__, *,
                 authenticator: Optional[pulumi.Input[str]] = None,
                 parent_flow_alias: Optional[pulumi.Input[str]] = None,
                 realm_id: Optional[pulumi.Input[str]] = None,
                 requirement: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Execution resources.
        :param pulumi.Input[str] authenticator: The name of the authenticator. This can be found by experimenting with the GUI and looking at HTTP requests within the network tab of your browser's development tools.
        :param pulumi.Input[str] parent_flow_alias: The alias of the flow this execution is attached to.
        :param pulumi.Input[str] realm_id: The realm the authentication execution exists in.
        :param pulumi.Input[str] requirement: The requirement setting, which can be one of `REQUIRED`, `ALTERNATIVE`, `OPTIONAL`, `CONDITIONAL`, or `DISABLED`. Defaults to `DISABLED`.
        """
        if authenticator is not None:
            pulumi.set(__self__, "authenticator", authenticator)
        if parent_flow_alias is not None:
            pulumi.set(__self__, "parent_flow_alias", parent_flow_alias)
        if realm_id is not None:
            pulumi.set(__self__, "realm_id", realm_id)
        if requirement is not None:
            pulumi.set(__self__, "requirement", requirement)

    @property
    @pulumi.getter
    def authenticator(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the authenticator. This can be found by experimenting with the GUI and looking at HTTP requests within the network tab of your browser's development tools.
        """
        return pulumi.get(self, "authenticator")

    @authenticator.setter
    def authenticator(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "authenticator", value)

    @property
    @pulumi.getter(name="parentFlowAlias")
    def parent_flow_alias(self) -> Optional[pulumi.Input[str]]:
        """
        The alias of the flow this execution is attached to.
        """
        return pulumi.get(self, "parent_flow_alias")

    @parent_flow_alias.setter
    def parent_flow_alias(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "parent_flow_alias", value)

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> Optional[pulumi.Input[str]]:
        """
        The realm the authentication execution exists in.
        """
        return pulumi.get(self, "realm_id")

    @realm_id.setter
    def realm_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "realm_id", value)

    @property
    @pulumi.getter
    def requirement(self) -> Optional[pulumi.Input[str]]:
        """
        The requirement setting, which can be one of `REQUIRED`, `ALTERNATIVE`, `OPTIONAL`, `CONDITIONAL`, or `DISABLED`. Defaults to `DISABLED`.
        """
        return pulumi.get(self, "requirement")

    @requirement.setter
    def requirement(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "requirement", value)


class Execution(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 authenticator: Optional[pulumi.Input[str]] = None,
                 parent_flow_alias: Optional[pulumi.Input[str]] = None,
                 realm_id: Optional[pulumi.Input[str]] = None,
                 requirement: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Allows for creating and managing an authentication execution within Keycloak.

        An authentication execution is an action that the user or service may or may not take when authenticating through an authentication
        flow.

        > Due to limitations in the Keycloak API, the ordering of authentication executions within a flow must be specified using `depends_on`. Authentication executions that are created first will appear first within the flow.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_keycloak as keycloak

        realm = keycloak.Realm("realm",
            realm="my-realm",
            enabled=True)
        flow = keycloak.authentication.Flow("flow",
            realm_id=realm.id,
            alias="my-flow-alias")
        # first execution
        execution_one = keycloak.authentication.Execution("executionOne",
            realm_id=realm.id,
            parent_flow_alias=flow.alias,
            authenticator="auth-cookie",
            requirement="ALTERNATIVE")
        # second execution
        execution_two = keycloak.authentication.Execution("executionTwo",
            realm_id=realm.id,
            parent_flow_alias=flow.alias,
            authenticator="identity-provider-redirector",
            requirement="ALTERNATIVE",
            opts=pulumi.ResourceOptions(depends_on=[execution_one]))
        ```

        ## Import

        Authentication executions can be imported using the formats`{{realmId}}/{{parentFlowAlias}}/{{authenticationExecutionId}}`. Examplebash

        ```sh
         $ pulumi import keycloak:authentication/execution:Execution execution_one my-realm/my-flow-alias/30559fcf-6fb8-45ea-8c46-2b86f46ebc17
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] authenticator: The name of the authenticator. This can be found by experimenting with the GUI and looking at HTTP requests within the network tab of your browser's development tools.
        :param pulumi.Input[str] parent_flow_alias: The alias of the flow this execution is attached to.
        :param pulumi.Input[str] realm_id: The realm the authentication execution exists in.
        :param pulumi.Input[str] requirement: The requirement setting, which can be one of `REQUIRED`, `ALTERNATIVE`, `OPTIONAL`, `CONDITIONAL`, or `DISABLED`. Defaults to `DISABLED`.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ExecutionArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Allows for creating and managing an authentication execution within Keycloak.

        An authentication execution is an action that the user or service may or may not take when authenticating through an authentication
        flow.

        > Due to limitations in the Keycloak API, the ordering of authentication executions within a flow must be specified using `depends_on`. Authentication executions that are created first will appear first within the flow.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_keycloak as keycloak

        realm = keycloak.Realm("realm",
            realm="my-realm",
            enabled=True)
        flow = keycloak.authentication.Flow("flow",
            realm_id=realm.id,
            alias="my-flow-alias")
        # first execution
        execution_one = keycloak.authentication.Execution("executionOne",
            realm_id=realm.id,
            parent_flow_alias=flow.alias,
            authenticator="auth-cookie",
            requirement="ALTERNATIVE")
        # second execution
        execution_two = keycloak.authentication.Execution("executionTwo",
            realm_id=realm.id,
            parent_flow_alias=flow.alias,
            authenticator="identity-provider-redirector",
            requirement="ALTERNATIVE",
            opts=pulumi.ResourceOptions(depends_on=[execution_one]))
        ```

        ## Import

        Authentication executions can be imported using the formats`{{realmId}}/{{parentFlowAlias}}/{{authenticationExecutionId}}`. Examplebash

        ```sh
         $ pulumi import keycloak:authentication/execution:Execution execution_one my-realm/my-flow-alias/30559fcf-6fb8-45ea-8c46-2b86f46ebc17
        ```

        :param str resource_name: The name of the resource.
        :param ExecutionArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ExecutionArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 authenticator: Optional[pulumi.Input[str]] = None,
                 parent_flow_alias: Optional[pulumi.Input[str]] = None,
                 realm_id: Optional[pulumi.Input[str]] = None,
                 requirement: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = _utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ExecutionArgs.__new__(ExecutionArgs)

            if authenticator is None and not opts.urn:
                raise TypeError("Missing required property 'authenticator'")
            __props__.__dict__["authenticator"] = authenticator
            if parent_flow_alias is None and not opts.urn:
                raise TypeError("Missing required property 'parent_flow_alias'")
            __props__.__dict__["parent_flow_alias"] = parent_flow_alias
            if realm_id is None and not opts.urn:
                raise TypeError("Missing required property 'realm_id'")
            __props__.__dict__["realm_id"] = realm_id
            __props__.__dict__["requirement"] = requirement
        super(Execution, __self__).__init__(
            'keycloak:authentication/execution:Execution',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            authenticator: Optional[pulumi.Input[str]] = None,
            parent_flow_alias: Optional[pulumi.Input[str]] = None,
            realm_id: Optional[pulumi.Input[str]] = None,
            requirement: Optional[pulumi.Input[str]] = None) -> 'Execution':
        """
        Get an existing Execution resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] authenticator: The name of the authenticator. This can be found by experimenting with the GUI and looking at HTTP requests within the network tab of your browser's development tools.
        :param pulumi.Input[str] parent_flow_alias: The alias of the flow this execution is attached to.
        :param pulumi.Input[str] realm_id: The realm the authentication execution exists in.
        :param pulumi.Input[str] requirement: The requirement setting, which can be one of `REQUIRED`, `ALTERNATIVE`, `OPTIONAL`, `CONDITIONAL`, or `DISABLED`. Defaults to `DISABLED`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ExecutionState.__new__(_ExecutionState)

        __props__.__dict__["authenticator"] = authenticator
        __props__.__dict__["parent_flow_alias"] = parent_flow_alias
        __props__.__dict__["realm_id"] = realm_id
        __props__.__dict__["requirement"] = requirement
        return Execution(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def authenticator(self) -> pulumi.Output[str]:
        """
        The name of the authenticator. This can be found by experimenting with the GUI and looking at HTTP requests within the network tab of your browser's development tools.
        """
        return pulumi.get(self, "authenticator")

    @property
    @pulumi.getter(name="parentFlowAlias")
    def parent_flow_alias(self) -> pulumi.Output[str]:
        """
        The alias of the flow this execution is attached to.
        """
        return pulumi.get(self, "parent_flow_alias")

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> pulumi.Output[str]:
        """
        The realm the authentication execution exists in.
        """
        return pulumi.get(self, "realm_id")

    @property
    @pulumi.getter
    def requirement(self) -> pulumi.Output[Optional[str]]:
        """
        The requirement setting, which can be one of `REQUIRED`, `ALTERNATIVE`, `OPTIONAL`, `CONDITIONAL`, or `DISABLED`. Defaults to `DISABLED`.
        """
        return pulumi.get(self, "requirement")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'GetUserRealmRolesResult',
    'AwaitableGetUserRealmRolesResult',
    'get_user_realm_roles',
    'get_user_realm_roles_output',
]

@pulumi.output_type
class GetUserRealmRolesResult:
    """
    A collection of values returned by getUserRealmRoles.
    """
    def __init__(__self__, id=None, realm_id=None, role_names=None, user_id=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if realm_id and not isinstance(realm_id, str):
            raise TypeError("Expected argument 'realm_id' to be a str")
        pulumi.set(__self__, "realm_id", realm_id)
        if role_names and not isinstance(role_names, list):
            raise TypeError("Expected argument 'role_names' to be a list")
        pulumi.set(__self__, "role_names", role_names)
        if user_id and not isinstance(user_id, str):
            raise TypeError("Expected argument 'user_id' to be a str")
        pulumi.set(__self__, "user_id", user_id)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> str:
        return pulumi.get(self, "realm_id")

    @property
    @pulumi.getter(name="roleNames")
    def role_names(self) -> Sequence[str]:
        """
        (Computed) A list of realm roles that belong to this user.
        """
        return pulumi.get(self, "role_names")

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> str:
        return pulumi.get(self, "user_id")


class AwaitableGetUserRealmRolesResult(GetUserRealmRolesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetUserRealmRolesResult(
            id=self.id,
            realm_id=self.realm_id,
            role_names=self.role_names,
            user_id=self.user_id)


def get_user_realm_roles(realm_id: Optional[str] = None,
                         user_id: Optional[str] = None,
                         opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetUserRealmRolesResult:
    """
    This data source can be used to fetch the realm roles of a user within Keycloak.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_keycloak as keycloak

    master_realm = keycloak.get_realm(realm="master")
    default_admin_user = keycloak.get_user(realm_id=master_realm.id,
        username="keycloak")
    user_realm_roles = keycloak.get_user_realm_roles(realm_id=master_realm.id,
        user_id=default_admin_user.id)
    pulumi.export("keycloakUserRoleNames", user_realm_roles.role_names)
    ```


    :param str realm_id: The realm this user belongs to.
    :param str user_id: The ID of the user to query realm roles for.
    """
    __args__ = dict()
    __args__['realmId'] = realm_id
    __args__['userId'] = user_id
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = _utilities.get_version()
    __ret__ = pulumi.runtime.invoke('keycloak:index/getUserRealmRoles:getUserRealmRoles', __args__, opts=opts, typ=GetUserRealmRolesResult).value

    return AwaitableGetUserRealmRolesResult(
        id=__ret__.id,
        realm_id=__ret__.realm_id,
        role_names=__ret__.role_names,
        user_id=__ret__.user_id)


@_utilities.lift_output_func(get_user_realm_roles)
def get_user_realm_roles_output(realm_id: Optional[pulumi.Input[str]] = None,
                                user_id: Optional[pulumi.Input[str]] = None,
                                opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetUserRealmRolesResult]:
    """
    This data source can be used to fetch the realm roles of a user within Keycloak.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_keycloak as keycloak

    master_realm = keycloak.get_realm(realm="master")
    default_admin_user = keycloak.get_user(realm_id=master_realm.id,
        username="keycloak")
    user_realm_roles = keycloak.get_user_realm_roles(realm_id=master_realm.id,
        user_id=default_admin_user.id)
    pulumi.export("keycloakUserRoleNames", user_realm_roles.role_names)
    ```


    :param str realm_id: The realm this user belongs to.
    :param str user_id: The ID of the user to query realm roles for.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['GroupMapperArgs', 'GroupMapper']

@pulumi.input_type
class GroupMapperArgs:
    def __init__(__self__, *,
                 group_name_ldap_attribute: pulumi.Input[str],
                 group_object_classes: pulumi.Input[Sequence[pulumi.Input[str]]],
                 ldap_groups_dn: pulumi.Input[str],
                 ldap_user_federation_id: pulumi.Input[str],
                 membership_ldap_attribute: pulumi.Input[str],
                 membership_user_ldap_attribute: pulumi.Input[str],
                 realm_id: pulumi.Input[str],
                 drop_non_existing_groups_during_sync: Optional[pulumi.Input[bool]] = None,
                 groups_ldap_filter: Optional[pulumi.Input[str]] = None,
                 groups_path: Optional[pulumi.Input[str]] = None,
                 ignore_missing_groups: Optional[pulumi.Input[bool]] = None,
                 mapped_group_attributes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 memberof_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 membership_attribute_type: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 preserve_group_inheritance: Optional[pulumi.Input[bool]] = None,
                 user_roles_retrieve_strategy: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a GroupMapper resource.
        :param pulumi.Input[str] group_name_ldap_attribute: The name of the LDAP attribute that is used in group objects for the name and RDN of the group. Typically `cn`.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_object_classes: List of strings representing the object classes for the group. Must contain at least one.
        :param pulumi.Input[str] ldap_groups_dn: The LDAP DN where groups can be found.
        :param pulumi.Input[str] ldap_user_federation_id: The ID of the LDAP user federation provider to attach this mapper to.
        :param pulumi.Input[str] membership_ldap_attribute: The name of the LDAP attribute that is used for membership mappings.
        :param pulumi.Input[str] membership_user_ldap_attribute: The name of the LDAP attribute on a user that is used for membership mappings.
        :param pulumi.Input[str] realm_id: The realm that this LDAP mapper will exist in.
        :param pulumi.Input[bool] drop_non_existing_groups_during_sync: When `true`, groups that no longer exist within LDAP will be dropped in Keycloak during sync. Defaults to `false`.
        :param pulumi.Input[str] groups_ldap_filter: When specified, adds an additional custom filter to be used when querying for groups. Must start with `(` and end with `)`.
        :param pulumi.Input[str] groups_path: Keycloak group path the LDAP groups are added to. For example if value `/Applications/App1` is used, then LDAP groups will be available in Keycloak under group `App1`, which is the child of top level group `Applications`. The configured group path must already exist in Keycloak when creating this mapper.
        :param pulumi.Input[bool] ignore_missing_groups: When `true`, missing groups in the hierarchy will be ignored.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] mapped_group_attributes: Array of strings representing attributes on the LDAP group which will be mapped to attributes on the Keycloak group.
        :param pulumi.Input[str] memberof_ldap_attribute: Specifies the name of the LDAP attribute on the LDAP user that contains the groups the user is a member of. Defaults to `memberOf`.
        :param pulumi.Input[str] membership_attribute_type: Can be one of `DN` or `UID`. Defaults to `DN`.
        :param pulumi.Input[str] mode: Can be one of `READ_ONLY`, `LDAP_ONLY` or `IMPORT`. Defaults to `READ_ONLY`.
        :param pulumi.Input[str] name: Display name of this mapper when displayed in the console.
        :param pulumi.Input[bool] preserve_group_inheritance: When `true`, group inheritance will be propagated from LDAP to Keycloak. When `false`, all LDAP groups will be propagated as top level groups within Keycloak.
        :param pulumi.Input[str] user_roles_retrieve_strategy: Can be one of `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`, `GET_GROUPS_FROM_USER_MEMBEROF_ATTRIBUTE`, or `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE_RECURSIVELY`. Defaults to `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`.
        """
        pulumi.set(__self__, "group_name_ldap_attribute", group_name_ldap_attribute)
        pulumi.set(__self__, "group_object_classes", group_object_classes)
        pulumi.set(__self__, "ldap_groups_dn", ldap_groups_dn)
        pulumi.set(__self__, "ldap_user_federation_id", ldap_user_federation_id)
        pulumi.set(__self__, "membership_ldap_attribute", membership_ldap_attribute)
        pulumi.set(__self__, "membership_user_ldap_attribute", membership_user_ldap_attribute)
        pulumi.set(__self__, "realm_id", realm_id)
        if drop_non_existing_groups_during_sync is not None:
            pulumi.set(__self__, "drop_non_existing_groups_during_sync", drop_non_existing_groups_during_sync)
        if groups_ldap_filter is not None:
            pulumi.set(__self__, "groups_ldap_filter", groups_ldap_filter)
        if groups_path is not None:
            pulumi.set(__self__, "groups_path", groups_path)
        if ignore_missing_groups is not None:
            pulumi.set(__self__, "ignore_missing_groups", ignore_missing_groups)
        if mapped_group_attributes is not None:
            pulumi.set(__self__, "mapped_group_attributes", mapped_group_attributes)
        if memberof_ldap_attribute is not None:
            pulumi.set(__self__, "memberof_ldap_attribute", memberof_ldap_attribute)
        if membership_attribute_type is not None:
            pulumi.set(__self__, "membership_attribute_type", membership_attribute_type)
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if preserve_group_inheritance is not None:
            pulumi.set(__self__, "preserve_group_inheritance", preserve_group_inheritance)
        if user_roles_retrieve_strategy is not None:
            pulumi.set(__self__, "user_roles_retrieve_strategy", user_roles_retrieve_strategy)

    @property
    @pulumi.getter(name="groupNameLdapAttribute")
    def group_name_ldap_attribute(self) -> pulumi.Input[str]:
        """
        The name of the LDAP attribute that is used in group objects for the name and RDN of the group. Typically `cn`.
        """
        return pulumi.get(self, "group_name_ldap_attribute")

    @group_name_ldap_attribute.setter
    def group_name_ldap_attribute(self, value: pulumi.Input[str]):
        pulumi.set(self, "group_name_ldap_attribute", value)

    @property
    @pulumi.getter(name="groupObjectClasses")
    def group_object_classes(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        """
        List of strings representing the object classes for the group. Must contain at least one.
        """
        return pulumi.get(self, "group_object_classes")

    @group_object_classes.setter
    def group_object_classes(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "group_object_classes", value)

    @property
    @pulumi.getter(name="ldapGroupsDn")
    def ldap_groups_dn(self) -> pulumi.Input[str]:
        """
        The LDAP DN where groups can be found.
        """
        return pulumi.get(self, "ldap_groups_dn")

    @ldap_groups_dn.setter
    def ldap_groups_dn(self, value: pulumi.Input[str]):
        pulumi.set(self, "ldap_groups_dn", value)

    @property
    @pulumi.getter(name="ldapUserFederationId")
    def ldap_user_federation_id(self) -> pulumi.Input[str]:
        """
        The ID of the LDAP user federation provider to attach this mapper to.
        """
        return pulumi.get(self, "ldap_user_federation_id")

    @ldap_user_federation_id.setter
    def ldap_user_federation_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "ldap_user_federation_id", value)

    @property
    @pulumi.getter(name="membershipLdapAttribute")
    def membership_ldap_attribute(self) -> pulumi.Input[str]:
        """
        The name of the LDAP attribute that is used for membership mappings.
        """
        return pulumi.get(self, "membership_ldap_attribute")

    @membership_ldap_attribute.setter
    def membership_ldap_attribute(self, value: pulumi.Input[str]):
        pulumi.set(self, "membership_ldap_attribute", value)

    @property
    @pulumi.getter(name="membershipUserLdapAttribute")
    def membership_user_ldap_attribute(self) -> pulumi.Input[str]:
        """
        The name of the LDAP attribute on a user that is used for membership mappings.
        """
        return pulumi.get(self, "membership_user_ldap_attribute")

    @membership_user_ldap_attribute.setter
    def membership_user_ldap_attribute(self, value: pulumi.Input[str]):
        pulumi.set(self, "membership_user_ldap_attribute", value)

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> pulumi.Input[str]:
        """
        The realm that this LDAP mapper will exist in.
        """
        return pulumi.get(self, "realm_id")

    @realm_id.setter
    def realm_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "realm_id", value)

    @property
    @pulumi.getter(name="dropNonExistingGroupsDuringSync")
    def drop_non_existing_groups_during_sync(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, groups that no longer exist within LDAP will be dropped in Keycloak during sync. Defaults to `false`.
        """
        return pulumi.get(self, "drop_non_existing_groups_during_sync")

    @drop_non_existing_groups_during_sync.setter
    def drop_non_existing_groups_during_sync(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "drop_non_existing_groups_during_sync", value)

    @property
    @pulumi.getter(name="groupsLdapFilter")
    def groups_ldap_filter(self) -> Optional[pulumi.Input[str]]:
        """
        When specified, adds an additional custom filter to be used when querying for groups. Must start with `(` and end with `)`.
        """
        return pulumi.get(self, "groups_ldap_filter")

    @groups_ldap_filter.setter
    def groups_ldap_filter(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "groups_ldap_filter", value)

    @property
    @pulumi.getter(name="groupsPath")
    def groups_path(self) -> Optional[pulumi.Input[str]]:
        """
        Keycloak group path the LDAP groups are added to. For example if value `/Applications/App1` is used, then LDAP groups will be available in Keycloak under group `App1`, which is the child of top level group `Applications`. The configured group path must already exist in Keycloak when creating this mapper.
        """
        return pulumi.get(self, "groups_path")

    @groups_path.setter
    def groups_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "groups_path", value)

    @property
    @pulumi.getter(name="ignoreMissingGroups")
    def ignore_missing_groups(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, missing groups in the hierarchy will be ignored.
        """
        return pulumi.get(self, "ignore_missing_groups")

    @ignore_missing_groups.setter
    def ignore_missing_groups(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ignore_missing_groups", value)

    @property
    @pulumi.getter(name="mappedGroupAttributes")
    def mapped_group_attributes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Array of strings representing attributes on the LDAP group which will be mapped to attributes on the Keycloak group.
        """
        return pulumi.get(self, "mapped_group_attributes")

    @mapped_group_attributes.setter
    def mapped_group_attributes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "mapped_group_attributes", value)

    @property
    @pulumi.getter(name="memberofLdapAttribute")
    def memberof_ldap_attribute(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the name of the LDAP attribute on the LDAP user that contains the groups the user is a member of. Defaults to `memberOf`.
        """
        return pulumi.get(self, "memberof_ldap_attribute")

    @memberof_ldap_attribute.setter
    def memberof_ldap_attribute(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "memberof_ldap_attribute", value)

    @property
    @pulumi.getter(name="membershipAttributeType")
    def membership_attribute_type(self) -> Optional[pulumi.Input[str]]:
        """
        Can be one of `DN` or `UID`. Defaults to `DN`.
        """
        return pulumi.get(self, "membership_attribute_type")

    @membership_attribute_type.setter
    def membership_attribute_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "membership_attribute_type", value)

    @property
    @pulumi.getter
    def mode(self) -> Optional[pulumi.Input[str]]:
        """
        Can be one of `READ_ONLY`, `LDAP_ONLY` or `IMPORT`. Defaults to `READ_ONLY`.
        """
        return pulumi.get(self, "mode")

    @mode.setter
    def mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mode", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Display name of this mapper when displayed in the console.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="preserveGroupInheritance")
    def preserve_group_inheritance(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, group inheritance will be propagated from LDAP to Keycloak. When `false`, all LDAP groups will be propagated as top level groups within Keycloak.
        """
        return pulumi.get(self, "preserve_group_inheritance")

    @preserve_group_inheritance.setter
    def preserve_group_inheritance(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "preserve_group_inheritance", value)

    @property
    @pulumi.getter(name="userRolesRetrieveStrategy")
    def user_roles_retrieve_strategy(self) -> Optional[pulumi.Input[str]]:
        """
        Can be one of `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`, `GET_GROUPS_FROM_USER_MEMBEROF_ATTRIBUTE`, or `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE_RECURSIVELY`. Defaults to `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`.
        """
        return pulumi.get(self, "user_roles_retrieve_strategy")

    @user_roles_retrieve_strategy.setter
    def user_roles_retrieve_strategy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_roles_retrieve_strategy", value)


@pulumi.input_type
class _GroupMapperState:
    def __init__(__self__, *,
                 drop_non_existing_groups_during_sync: Optional[pulumi.Input[bool]] = None,
                 group_name_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 group_object_classes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 groups_ldap_filter: Optional[pulumi.Input[str]] = None,
                 groups_path: Optional[pulumi.Input[str]] = None,
                 ignore_missing_groups: Optional[pulumi.Input[bool]] = None,
                 ldap_groups_dn: Optional[pulumi.Input[str]] = None,
                 ldap_user_federation_id: Optional[pulumi.Input[str]] = None,
                 mapped_group_attributes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 memberof_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 membership_attribute_type: Optional[pulumi.Input[str]] = None,
                 membership_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 membership_user_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 preserve_group_inheritance: Optional[pulumi.Input[bool]] = None,
                 realm_id: Optional[pulumi.Input[str]] = None,
                 user_roles_retrieve_strategy: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering GroupMapper resources.
        :param pulumi.Input[bool] drop_non_existing_groups_during_sync: When `true`, groups that no longer exist within LDAP will be dropped in Keycloak during sync. Defaults to `false`.
        :param pulumi.Input[str] group_name_ldap_attribute: The name of the LDAP attribute that is used in group objects for the name and RDN of the group. Typically `cn`.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_object_classes: List of strings representing the object classes for the group. Must contain at least one.
        :param pulumi.Input[str] groups_ldap_filter: When specified, adds an additional custom filter to be used when querying for groups. Must start with `(` and end with `)`.
        :param pulumi.Input[str] groups_path: Keycloak group path the LDAP groups are added to. For example if value `/Applications/App1` is used, then LDAP groups will be available in Keycloak under group `App1`, which is the child of top level group `Applications`. The configured group path must already exist in Keycloak when creating this mapper.
        :param pulumi.Input[bool] ignore_missing_groups: When `true`, missing groups in the hierarchy will be ignored.
        :param pulumi.Input[str] ldap_groups_dn: The LDAP DN where groups can be found.
        :param pulumi.Input[str] ldap_user_federation_id: The ID of the LDAP user federation provider to attach this mapper to.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] mapped_group_attributes: Array of strings representing attributes on the LDAP group which will be mapped to attributes on the Keycloak group.
        :param pulumi.Input[str] memberof_ldap_attribute: Specifies the name of the LDAP attribute on the LDAP user that contains the groups the user is a member of. Defaults to `memberOf`.
        :param pulumi.Input[str] membership_attribute_type: Can be one of `DN` or `UID`. Defaults to `DN`.
        :param pulumi.Input[str] membership_ldap_attribute: The name of the LDAP attribute that is used for membership mappings.
        :param pulumi.Input[str] membership_user_ldap_attribute: The name of the LDAP attribute on a user that is used for membership mappings.
        :param pulumi.Input[str] mode: Can be one of `READ_ONLY`, `LDAP_ONLY` or `IMPORT`. Defaults to `READ_ONLY`.
        :param pulumi.Input[str] name: Display name of this mapper when displayed in the console.
        :param pulumi.Input[bool] preserve_group_inheritance: When `true`, group inheritance will be propagated from LDAP to Keycloak. When `false`, all LDAP groups will be propagated as top level groups within Keycloak.
        :param pulumi.Input[str] realm_id: The realm that this LDAP mapper will exist in.
        :param pulumi.Input[str] user_roles_retrieve_strategy: Can be one of `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`, `GET_GROUPS_FROM_USER_MEMBEROF_ATTRIBUTE`, or `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE_RECURSIVELY`. Defaults to `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`.
        """
        if drop_non_existing_groups_during_sync is not None:
            pulumi.set(__self__, "drop_non_existing_groups_during_sync", drop_non_existing_groups_during_sync)
        if group_name_ldap_attribute is not None:
            pulumi.set(__self__, "group_name_ldap_attribute", group_name_ldap_attribute)
        if group_object_classes is not None:
            pulumi.set(__self__, "group_object_classes", group_object_classes)
        if groups_ldap_filter is not None:
            pulumi.set(__self__, "groups_ldap_filter", groups_ldap_filter)
        if groups_path is not None:
            pulumi.set(__self__, "groups_path", groups_path)
        if ignore_missing_groups is not None:
            pulumi.set(__self__, "ignore_missing_groups", ignore_missing_groups)
        if ldap_groups_dn is not None:
            pulumi.set(__self__, "ldap_groups_dn", ldap_groups_dn)
        if ldap_user_federation_id is not None:
            pulumi.set(__self__, "ldap_user_federation_id", ldap_user_federation_id)
        if mapped_group_attributes is not None:
            pulumi.set(__self__, "mapped_group_attributes", mapped_group_attributes)
        if memberof_ldap_attribute is not None:
            pulumi.set(__self__, "memberof_ldap_attribute", memberof_ldap_attribute)
        if membership_attribute_type is not None:
            pulumi.set(__self__, "membership_attribute_type", membership_attribute_type)
        if membership_ldap_attribute is not None:
            pulumi.set(__self__, "membership_ldap_attribute", membership_ldap_attribute)
        if membership_user_ldap_attribute is not None:
            pulumi.set(__self__, "membership_user_ldap_attribute", membership_user_ldap_attribute)
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if preserve_group_inheritance is not None:
            pulumi.set(__self__, "preserve_group_inheritance", preserve_group_inheritance)
        if realm_id is not None:
            pulumi.set(__self__, "realm_id", realm_id)
        if user_roles_retrieve_strategy is not None:
            pulumi.set(__self__, "user_roles_retrieve_strategy", user_roles_retrieve_strategy)

    @property
    @pulumi.getter(name="dropNonExistingGroupsDuringSync")
    def drop_non_existing_groups_during_sync(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, groups that no longer exist within LDAP will be dropped in Keycloak during sync. Defaults to `false`.
        """
        return pulumi.get(self, "drop_non_existing_groups_during_sync")

    @drop_non_existing_groups_during_sync.setter
    def drop_non_existing_groups_during_sync(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "drop_non_existing_groups_during_sync", value)

    @property
    @pulumi.getter(name="groupNameLdapAttribute")
    def group_name_ldap_attribute(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the LDAP attribute that is used in group objects for the name and RDN of the group. Typically `cn`.
        """
        return pulumi.get(self, "group_name_ldap_attribute")

    @group_name_ldap_attribute.setter
    def group_name_ldap_attribute(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "group_name_ldap_attribute", value)

    @property
    @pulumi.getter(name="groupObjectClasses")
    def group_object_classes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        List of strings representing the object classes for the group. Must contain at least one.
        """
        return pulumi.get(self, "group_object_classes")

    @group_object_classes.setter
    def group_object_classes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "group_object_classes", value)

    @property
    @pulumi.getter(name="groupsLdapFilter")
    def groups_ldap_filter(self) -> Optional[pulumi.Input[str]]:
        """
        When specified, adds an additional custom filter to be used when querying for groups. Must start with `(` and end with `)`.
        """
        return pulumi.get(self, "groups_ldap_filter")

    @groups_ldap_filter.setter
    def groups_ldap_filter(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "groups_ldap_filter", value)

    @property
    @pulumi.getter(name="groupsPath")
    def groups_path(self) -> Optional[pulumi.Input[str]]:
        """
        Keycloak group path the LDAP groups are added to. For example if value `/Applications/App1` is used, then LDAP groups will be available in Keycloak under group `App1`, which is the child of top level group `Applications`. The configured group path must already exist in Keycloak when creating this mapper.
        """
        return pulumi.get(self, "groups_path")

    @groups_path.setter
    def groups_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "groups_path", value)

    @property
    @pulumi.getter(name="ignoreMissingGroups")
    def ignore_missing_groups(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, missing groups in the hierarchy will be ignored.
        """
        return pulumi.get(self, "ignore_missing_groups")

    @ignore_missing_groups.setter
    def ignore_missing_groups(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "ignore_missing_groups", value)

    @property
    @pulumi.getter(name="ldapGroupsDn")
    def ldap_groups_dn(self) -> Optional[pulumi.Input[str]]:
        """
        The LDAP DN where groups can be found.
        """
        return pulumi.get(self, "ldap_groups_dn")

    @ldap_groups_dn.setter
    def ldap_groups_dn(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ldap_groups_dn", value)

    @property
    @pulumi.getter(name="ldapUserFederationId")
    def ldap_user_federation_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the LDAP user federation provider to attach this mapper to.
        """
        return pulumi.get(self, "ldap_user_federation_id")

    @ldap_user_federation_id.setter
    def ldap_user_federation_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ldap_user_federation_id", value)

    @property
    @pulumi.getter(name="mappedGroupAttributes")
    def mapped_group_attributes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Array of strings representing attributes on the LDAP group which will be mapped to attributes on the Keycloak group.
        """
        return pulumi.get(self, "mapped_group_attributes")

    @mapped_group_attributes.setter
    def mapped_group_attributes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "mapped_group_attributes", value)

    @property
    @pulumi.getter(name="memberofLdapAttribute")
    def memberof_ldap_attribute(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the name of the LDAP attribute on the LDAP user that contains the groups the user is a member of. Defaults to `memberOf`.
        """
        return pulumi.get(self, "memberof_ldap_attribute")

    @memberof_ldap_attribute.setter
    def memberof_ldap_attribute(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "memberof_ldap_attribute", value)

    @property
    @pulumi.getter(name="membershipAttributeType")
    def membership_attribute_type(self) -> Optional[pulumi.Input[str]]:
        """
        Can be one of `DN` or `UID`. Defaults to `DN`.
        """
        return pulumi.get(self, "membership_attribute_type")

    @membership_attribute_type.setter
    def membership_attribute_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "membership_attribute_type", value)

    @property
    @pulumi.getter(name="membershipLdapAttribute")
    def membership_ldap_attribute(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the LDAP attribute that is used for membership mappings.
        """
        return pulumi.get(self, "membership_ldap_attribute")

    @membership_ldap_attribute.setter
    def membership_ldap_attribute(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "membership_ldap_attribute", value)

    @property
    @pulumi.getter(name="membershipUserLdapAttribute")
    def membership_user_ldap_attribute(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the LDAP attribute on a user that is used for membership mappings.
        """
        return pulumi.get(self, "membership_user_ldap_attribute")

    @membership_user_ldap_attribute.setter
    def membership_user_ldap_attribute(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "membership_user_ldap_attribute", value)

    @property
    @pulumi.getter
    def mode(self) -> Optional[pulumi.Input[str]]:
        """
        Can be one of `READ_ONLY`, `LDAP_ONLY` or `IMPORT`. Defaults to `READ_ONLY`.
        """
        return pulumi.get(self, "mode")

    @mode.setter
    def mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mode", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Display name of this mapper when displayed in the console.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="preserveGroupInheritance")
    def preserve_group_inheritance(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, group inheritance will be propagated from LDAP to Keycloak. When `false`, all LDAP groups will be propagated as top level groups within Keycloak.
        """
        return pulumi.get(self, "preserve_group_inheritance")

    @preserve_group_inheritance.setter
    def preserve_group_inheritance(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "preserve_group_inheritance", value)

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> Optional[pulumi.Input[str]]:
        """
        The realm that this LDAP mapper will exist in.
        """
        return pulumi.get(self, "realm_id")

    @realm_id.setter
    def realm_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "realm_id", value)

    @property
    @pulumi.getter(name="userRolesRetrieveStrategy")
    def user_roles_retrieve_strategy(self) -> Optional[pulumi.Input[str]]:
        """
        Can be one of `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`, `GET_GROUPS_FROM_USER_MEMBEROF_ATTRIBUTE`, or `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE_RECURSIVELY`. Defaults to `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`.
        """
        return pulumi.get(self, "user_roles_retrieve_strategy")

    @user_roles_retrieve_strategy.setter
    def user_roles_retrieve_strategy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_roles_retrieve_strategy", value)


class GroupMapper(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 drop_non_existing_groups_during_sync: Optional[pulumi.Input[bool]] = None,
                 group_name_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 group_object_classes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 groups_ldap_filter: Optional[pulumi.Input[str]] = None,
                 groups_path: Optional[pulumi.Input[str]] = None,
                 ignore_missing_groups: Optional[pulumi.Input[bool]] = None,
                 ldap_groups_dn: Optional[pulumi.Input[str]] = None,
                 ldap_user_federation_id: Optional[pulumi.Input[str]] = None,
                 mapped_group_attributes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 memberof_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 membership_attribute_type: Optional[pulumi.Input[str]] = None,
                 membership_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 membership_user_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 preserve_group_inheritance: Optional[pulumi.Input[bool]] = None,
                 realm_id: Optional[pulumi.Input[str]] = None,
                 user_roles_retrieve_strategy: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Allows for creating and managing group mappers for Keycloak users federated via LDAP.

        The LDAP group mapper can be used to map an LDAP user's groups from some DN to Keycloak groups. This group mapper will also
        create the groups within Keycloak if they do not already exist.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_keycloak as keycloak

        realm = keycloak.Realm("realm",
            realm="my-realm",
            enabled=True)
        ldap_user_federation = keycloak.ldap.UserFederation("ldapUserFederation",
            realm_id=realm.id,
            username_ldap_attribute="cn",
            rdn_ldap_attribute="cn",
            uuid_ldap_attribute="entryDN",
            user_object_classes=[
                "simpleSecurityObject",
                "organizationalRole",
            ],
            connection_url="ldap://openldap",
            users_dn="dc=example,dc=org",
            bind_dn="cn=admin,dc=example,dc=org",
            bind_credential="admin")
        ldap_group_mapper = keycloak.ldap.GroupMapper("ldapGroupMapper",
            realm_id=realm.id,
            ldap_user_federation_id=ldap_user_federation.id,
            ldap_groups_dn="dc=example,dc=org",
            group_name_ldap_attribute="cn",
            group_object_classes=["groupOfNames"],
            membership_attribute_type="DN",
            membership_ldap_attribute="member",
            membership_user_ldap_attribute="cn",
            memberof_ldap_attribute="memberOf")
        ```

        ## Import

        LDAP mappers can be imported using the format `{{realm_id}}/{{ldap_user_federation_id}}/{{ldap_mapper_id}}`. The ID of the LDAP user federation provider and the mapper can be found within the Keycloak GUI, and they are typically GUIDs. Examplebash

        ```sh
         $ pulumi import keycloak:ldap/groupMapper:GroupMapper ldap_group_mapper my-realm/af2a6ca3-e4d7-49c3-b08b-1b3c70b4b860/3d923ece-1a91-4bf7-adaf-3b82f2a12b67
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] drop_non_existing_groups_during_sync: When `true`, groups that no longer exist within LDAP will be dropped in Keycloak during sync. Defaults to `false`.
        :param pulumi.Input[str] group_name_ldap_attribute: The name of the LDAP attribute that is used in group objects for the name and RDN of the group. Typically `cn`.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_object_classes: List of strings representing the object classes for the group. Must contain at least one.
        :param pulumi.Input[str] groups_ldap_filter: When specified, adds an additional custom filter to be used when querying for groups. Must start with `(` and end with `)`.
        :param pulumi.Input[str] groups_path: Keycloak group path the LDAP groups are added to. For example if value `/Applications/App1` is used, then LDAP groups will be available in Keycloak under group `App1`, which is the child of top level group `Applications`. The configured group path must already exist in Keycloak when creating this mapper.
        :param pulumi.Input[bool] ignore_missing_groups: When `true`, missing groups in the hierarchy will be ignored.
        :param pulumi.Input[str] ldap_groups_dn: The LDAP DN where groups can be found.
        :param pulumi.Input[str] ldap_user_federation_id: The ID of the LDAP user federation provider to attach this mapper to.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] mapped_group_attributes: Array of strings representing attributes on the LDAP group which will be mapped to attributes on the Keycloak group.
        :param pulumi.Input[str] memberof_ldap_attribute: Specifies the name of the LDAP attribute on the LDAP user that contains the groups the user is a member of. Defaults to `memberOf`.
        :param pulumi.Input[str] membership_attribute_type: Can be one of `DN` or `UID`. Defaults to `DN`.
        :param pulumi.Input[str] membership_ldap_attribute: The name of the LDAP attribute that is used for membership mappings.
        :param pulumi.Input[str] membership_user_ldap_attribute: The name of the LDAP attribute on a user that is used for membership mappings.
        :param pulumi.Input[str] mode: Can be one of `READ_ONLY`, `LDAP_ONLY` or `IMPORT`. Defaults to `READ_ONLY`.
        :param pulumi.Input[str] name: Display name of this mapper when displayed in the console.
        :param pulumi.Input[bool] preserve_group_inheritance: When `true`, group inheritance will be propagated from LDAP to Keycloak. When `false`, all LDAP groups will be propagated as top level groups within Keycloak.
        :param pulumi.Input[str] realm_id: The realm that this LDAP mapper will exist in.
        :param pulumi.Input[str] user_roles_retrieve_strategy: Can be one of `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`, `GET_GROUPS_FROM_USER_MEMBEROF_ATTRIBUTE`, or `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE_RECURSIVELY`. Defaults to `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: GroupMapperArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Allows for creating and managing group mappers for Keycloak users federated via LDAP.

        The LDAP group mapper can be used to map an LDAP user's groups from some DN to Keycloak groups. This group mapper will also
        create the groups within Keycloak if they do not already exist.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_keycloak as keycloak

        realm = keycloak.Realm("realm",
            realm="my-realm",
            enabled=True)
        ldap_user_federation = keycloak.ldap.UserFederation("ldapUserFederation",
            realm_id=realm.id,
            username_ldap_attribute="cn",
            rdn_ldap_attribute="cn",
            uuid_ldap_attribute="entryDN",
            user_object_classes=[
                "simpleSecurityObject",
                "organizationalRole",
            ],
            connection_url="ldap://openldap",
            users_dn="dc=example,dc=org",
            bind_dn="cn=admin,dc=example,dc=org",
            bind_credential="admin")
        ldap_group_mapper = keycloak.ldap.GroupMapper("ldapGroupMapper",
            realm_id=realm.id,
            ldap_user_federation_id=ldap_user_federation.id,
            ldap_groups_dn="dc=example,dc=org",
            group_name_ldap_attribute="cn",
            group_object_classes=["groupOfNames"],
            membership_attribute_type="DN",
            membership_ldap_attribute="member",
            membership_user_ldap_attribute="cn",
            memberof_ldap_attribute="memberOf")
        ```

        ## Import

        LDAP mappers can be imported using the format `{{realm_id}}/{{ldap_user_federation_id}}/{{ldap_mapper_id}}`. The ID of the LDAP user federation provider and the mapper can be found within the Keycloak GUI, and they are typically GUIDs. Examplebash

        ```sh
         $ pulumi import keycloak:ldap/groupMapper:GroupMapper ldap_group_mapper my-realm/af2a6ca3-e4d7-49c3-b08b-1b3c70b4b860/3d923ece-1a91-4bf7-adaf-3b82f2a12b67
        ```

        :param str resource_name: The name of the resource.
        :param GroupMapperArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(GroupMapperArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 drop_non_existing_groups_during_sync: Optional[pulumi.Input[bool]] = None,
                 group_name_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 group_object_classes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 groups_ldap_filter: Optional[pulumi.Input[str]] = None,
                 groups_path: Optional[pulumi.Input[str]] = None,
                 ignore_missing_groups: Optional[pulumi.Input[bool]] = None,
                 ldap_groups_dn: Optional[pulumi.Input[str]] = None,
                 ldap_user_federation_id: Optional[pulumi.Input[str]] = None,
                 mapped_group_attributes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 memberof_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 membership_attribute_type: Optional[pulumi.Input[str]] = None,
                 membership_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 membership_user_ldap_attribute: Optional[pulumi.Input[str]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 preserve_group_inheritance: Optional[pulumi.Input[bool]] = None,
                 realm_id: Optional[pulumi.Input[str]] = None,
                 user_roles_retrieve_strategy: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = _utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = GroupMapperArgs.__new__(GroupMapperArgs)

            __props__.__dict__["drop_non_existing_groups_during_sync"] = drop_non_existing_groups_during_sync
            if group_name_ldap_attribute is None and not opts.urn:
                raise TypeError("Missing required property 'group_name_ldap_attribute'")
            __props__.__dict__["group_name_ldap_attribute"] = group_name_ldap_attribute
            if group_object_classes is None and not opts.urn:
                raise TypeError("Missing required property 'group_object_classes'")
            __props__.__dict__["group_object_classes"] = group_object_classes
            __props__.__dict__["groups_ldap_filter"] = groups_ldap_filter
            __props__.__dict__["groups_path"] = groups_path
            __props__.__dict__["ignore_missing_groups"] = ignore_missing_groups
            if ldap_groups_dn is None and not opts.urn:
                raise TypeError("Missing required property 'ldap_groups_dn'")
            __props__.__dict__["ldap_groups_dn"] = ldap_groups_dn
            if ldap_user_federation_id is None and not opts.urn:
                raise TypeError("Missing required property 'ldap_user_federation_id'")
            __props__.__dict__["ldap_user_federation_id"] = ldap_user_federation_id
            __props__.__dict__["mapped_group_attributes"] = mapped_group_attributes
            __props__.__dict__["memberof_ldap_attribute"] = memberof_ldap_attribute
            __props__.__dict__["membership_attribute_type"] = membership_attribute_type
            if membership_ldap_attribute is None and not opts.urn:
                raise TypeError("Missing required property 'membership_ldap_attribute'")
            __props__.__dict__["membership_ldap_attribute"] = membership_ldap_attribute
            if membership_user_ldap_attribute is None and not opts.urn:
                raise TypeError("Missing required property 'membership_user_ldap_attribute'")
            __props__.__dict__["membership_user_ldap_attribute"] = membership_user_ldap_attribute
            __props__.__dict__["mode"] = mode
            __props__.__dict__["name"] = name
            __props__.__dict__["preserve_group_inheritance"] = preserve_group_inheritance
            if realm_id is None and not opts.urn:
                raise TypeError("Missing required property 'realm_id'")
            __props__.__dict__["realm_id"] = realm_id
            __props__.__dict__["user_roles_retrieve_strategy"] = user_roles_retrieve_strategy
        super(GroupMapper, __self__).__init__(
            'keycloak:ldap/groupMapper:GroupMapper',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            drop_non_existing_groups_during_sync: Optional[pulumi.Input[bool]] = None,
            group_name_ldap_attribute: Optional[pulumi.Input[str]] = None,
            group_object_classes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            groups_ldap_filter: Optional[pulumi.Input[str]] = None,
            groups_path: Optional[pulumi.Input[str]] = None,
            ignore_missing_groups: Optional[pulumi.Input[bool]] = None,
            ldap_groups_dn: Optional[pulumi.Input[str]] = None,
            ldap_user_federation_id: Optional[pulumi.Input[str]] = None,
            mapped_group_attributes: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            memberof_ldap_attribute: Optional[pulumi.Input[str]] = None,
            membership_attribute_type: Optional[pulumi.Input[str]] = None,
            membership_ldap_attribute: Optional[pulumi.Input[str]] = None,
            membership_user_ldap_attribute: Optional[pulumi.Input[str]] = None,
            mode: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            preserve_group_inheritance: Optional[pulumi.Input[bool]] = None,
            realm_id: Optional[pulumi.Input[str]] = None,
            user_roles_retrieve_strategy: Optional[pulumi.Input[str]] = None) -> 'GroupMapper':
        """
        Get an existing GroupMapper resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] drop_non_existing_groups_during_sync: When `true`, groups that no longer exist within LDAP will be dropped in Keycloak during sync. Defaults to `false`.
        :param pulumi.Input[str] group_name_ldap_attribute: The name of the LDAP attribute that is used in group objects for the name and RDN of the group. Typically `cn`.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_object_classes: List of strings representing the object classes for the group. Must contain at least one.
        :param pulumi.Input[str] groups_ldap_filter: When specified, adds an additional custom filter to be used when querying for groups. Must start with `(` and end with `)`.
        :param pulumi.Input[str] groups_path: Keycloak group path the LDAP groups are added to. For example if value `/Applications/App1` is used, then LDAP groups will be available in Keycloak under group `App1`, which is the child of top level group `Applications`. The configured group path must already exist in Keycloak when creating this mapper.
        :param pulumi.Input[bool] ignore_missing_groups: When `true`, missing groups in the hierarchy will be ignored.
        :param pulumi.Input[str] ldap_groups_dn: The LDAP DN where groups can be found.
        :param pulumi.Input[str] ldap_user_federation_id: The ID of the LDAP user federation provider to attach this mapper to.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] mapped_group_attributes: Array of strings representing attributes on the LDAP group which will be mapped to attributes on the Keycloak group.
        :param pulumi.Input[str] memberof_ldap_attribute: Specifies the name of the LDAP attribute on the LDAP user that contains the groups the user is a member of. Defaults to `memberOf`.
        :param pulumi.Input[str] membership_attribute_type: Can be one of `DN` or `UID`. Defaults to `DN`.
        :param pulumi.Input[str] membership_ldap_attribute: The name of the LDAP attribute that is used for membership mappings.
        :param pulumi.Input[str] membership_user_ldap_attribute: The name of the LDAP attribute on a user that is used for membership mappings.
        :param pulumi.Input[str] mode: Can be one of `READ_ONLY`, `LDAP_ONLY` or `IMPORT`. Defaults to `READ_ONLY`.
        :param pulumi.Input[str] name: Display name of this mapper when displayed in the console.
        :param pulumi.Input[bool] preserve_group_inheritance: When `true`, group inheritance will be propagated from LDAP to Keycloak. When `false`, all LDAP groups will be propagated as top level groups within Keycloak.
        :param pulumi.Input[str] realm_id: The realm that this LDAP mapper will exist in.
        :param pulumi.Input[str] user_roles_retrieve_strategy: Can be one of `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`, `GET_GROUPS_FROM_USER_MEMBEROF_ATTRIBUTE`, or `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE_RECURSIVELY`. Defaults to `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _GroupMapperState.__new__(_GroupMapperState)

        __props__.__dict__["drop_non_existing_groups_during_sync"] = drop_non_existing_groups_during_sync
        __props__.__dict__["group_name_ldap_attribute"] = group_name_ldap_attribute
        __props__.__dict__["group_object_classes"] = group_object_classes
        __props__.__dict__["groups_ldap_filter"] = groups_ldap_filter
        __props__.__dict__["groups_path"] = groups_path
        __props__.__dict__["ignore_missing_groups"] = ignore_missing_groups
        __props__.__dict__["ldap_groups_dn"] = ldap_groups_dn
        __props__.__dict__["ldap_user_federation_id"] = ldap_user_federation_id
        __props__.__dict__["mapped_group_attributes"] = mapped_group_attributes
        __props__.__dict__["memberof_ldap_attribute"] = memberof_ldap_attribute
        __props__.__dict__["membership_attribute_type"] = membership_attribute_type
        __props__.__dict__["membership_ldap_attribute"] = membership_ldap_attribute
        __props__.__dict__["membership_user_ldap_attribute"] = membership_user_ldap_attribute
        __props__.__dict__["mode"] = mode
        __props__.__dict__["name"] = name
        __props__.__dict__["preserve_group_inheritance"] = preserve_group_inheritance
        __props__.__dict__["realm_id"] = realm_id
        __props__.__dict__["user_roles_retrieve_strategy"] = user_roles_retrieve_strategy
        return GroupMapper(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="dropNonExistingGroupsDuringSync")
    def drop_non_existing_groups_during_sync(self) -> pulumi.Output[Optional[bool]]:
        """
        When `true`, groups that no longer exist within LDAP will be dropped in Keycloak during sync. Defaults to `false`.
        """
        return pulumi.get(self, "drop_non_existing_groups_during_sync")

    @property
    @pulumi.getter(name="groupNameLdapAttribute")
    def group_name_ldap_attribute(self) -> pulumi.Output[str]:
        """
        The name of the LDAP attribute that is used in group objects for the name and RDN of the group. Typically `cn`.
        """
        return pulumi.get(self, "group_name_ldap_attribute")

    @property
    @pulumi.getter(name="groupObjectClasses")
    def group_object_classes(self) -> pulumi.Output[Sequence[str]]:
        """
        List of strings representing the object classes for the group. Must contain at least one.
        """
        return pulumi.get(self, "group_object_classes")

    @property
    @pulumi.getter(name="groupsLdapFilter")
    def groups_ldap_filter(self) -> pulumi.Output[Optional[str]]:
        """
        When specified, adds an additional custom filter to be used when querying for groups. Must start with `(` and end with `)`.
        """
        return pulumi.get(self, "groups_ldap_filter")

    @property
    @pulumi.getter(name="groupsPath")
    def groups_path(self) -> pulumi.Output[str]:
        """
        Keycloak group path the LDAP groups are added to. For example if value `/Applications/App1` is used, then LDAP groups will be available in Keycloak under group `App1`, which is the child of top level group `Applications`. The configured group path must already exist in Keycloak when creating this mapper.
        """
        return pulumi.get(self, "groups_path")

    @property
    @pulumi.getter(name="ignoreMissingGroups")
    def ignore_missing_groups(self) -> pulumi.Output[Optional[bool]]:
        """
        When `true`, missing groups in the hierarchy will be ignored.
        """
        return pulumi.get(self, "ignore_missing_groups")

    @property
    @pulumi.getter(name="ldapGroupsDn")
    def ldap_groups_dn(self) -> pulumi.Output[str]:
        """
        The LDAP DN where groups can be found.
        """
        return pulumi.get(self, "ldap_groups_dn")

    @property
    @pulumi.getter(name="ldapUserFederationId")
    def ldap_user_federation_id(self) -> pulumi.Output[str]:
        """
        The ID of the LDAP user federation provider to attach this mapper to.
        """
        return pulumi.get(self, "ldap_user_federation_id")

    @property
    @pulumi.getter(name="mappedGroupAttributes")
    def mapped_group_attributes(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Array of strings representing attributes on the LDAP group which will be mapped to attributes on the Keycloak group.
        """
        return pulumi.get(self, "mapped_group_attributes")

    @property
    @pulumi.getter(name="memberofLdapAttribute")
    def memberof_ldap_attribute(self) -> pulumi.Output[Optional[str]]:
        """
        Specifies the name of the LDAP attribute on the LDAP user that contains the groups the user is a member of. Defaults to `memberOf`.
        """
        return pulumi.get(self, "memberof_ldap_attribute")

    @property
    @pulumi.getter(name="membershipAttributeType")
    def membership_attribute_type(self) -> pulumi.Output[Optional[str]]:
        """
        Can be one of `DN` or `UID`. Defaults to `DN`.
        """
        return pulumi.get(self, "membership_attribute_type")

    @property
    @pulumi.getter(name="membershipLdapAttribute")
    def membership_ldap_attribute(self) -> pulumi.Output[str]:
        """
        The name of the LDAP attribute that is used for membership mappings.
        """
        return pulumi.get(self, "membership_ldap_attribute")

    @property
    @pulumi.getter(name="membershipUserLdapAttribute")
    def membership_user_ldap_attribute(self) -> pulumi.Output[str]:
        """
        The name of the LDAP attribute on a user that is used for membership mappings.
        """
        return pulumi.get(self, "membership_user_ldap_attribute")

    @property
    @pulumi.getter
    def mode(self) -> pulumi.Output[Optional[str]]:
        """
        Can be one of `READ_ONLY`, `LDAP_ONLY` or `IMPORT`. Defaults to `READ_ONLY`.
        """
        return pulumi.get(self, "mode")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        Display name of this mapper when displayed in the console.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="preserveGroupInheritance")
    def preserve_group_inheritance(self) -> pulumi.Output[Optional[bool]]:
        """
        When `true`, group inheritance will be propagated from LDAP to Keycloak. When `false`, all LDAP groups will be propagated as top level groups within Keycloak.
        """
        return pulumi.get(self, "preserve_group_inheritance")

    @property
    @pulumi.getter(name="realmId")
    def realm_id(self) -> pulumi.Output[str]:
        """
        The realm that this LDAP mapper will exist in.
        """
        return pulumi.get(self, "realm_id")

    @property
    @pulumi.getter(name="userRolesRetrieveStrategy")
    def user_roles_retrieve_strategy(self) -> pulumi.Output[Optional[str]]:
        """
        Can be one of `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`, `GET_GROUPS_FROM_USER_MEMBEROF_ATTRIBUTE`, or `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE_RECURSIVELY`. Defaults to `LOAD_GROUPS_BY_MEMBER_ATTRIBUTE`.
        """
        return pulumi.get(self, "user_roles_retrieve_strategy")

